
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.gazetteer.idec.drivers;
import es.gva.cit.catalogClient.metadataXML.XMLNode;
import es.gva.cit.catalogClient.protocols.HTTPGetProtocol;
import es.gva.cit.catalogClient.protocols.HTTPPostProtocol;
import es.gva.cit.catalogClient.utils.Strings;
import es.gva.cit.gazetteer.drivers.AsbtractGazetteerServiceDriver;
import es.gva.cit.gazetteer.idec.parsers.IdecCapabilitiesParser;
import es.gva.cit.gazetteer.idec.parsers.IdecFeatureParser;
import es.gva.cit.gazetteer.querys.Feature;
import es.gva.cit.gazetteer.querys.Query;
import es.gva.cit.gazetteer.querys.ThesaurusName;
import java.net.URL;
import java.util.Collection;
import org.apache.commons.httpclient.NameValuePair;

/**
 * This class connects with the IDEC gazetteer service
 * 
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 */
public class IDECGazetteerServiceDriver extends AsbtractGazetteerServiceDriver {
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.IGazetteerServiceDriver#getCapabilities(java.net.URL)
     */

/**
 * 
 * 
 * 
 * @return 
 * @param url 
 */
    public Collection getCapabilities(URL url) {        
        Collection nodes = new java.util.ArrayList();
        System.out.println("**************GET*************");
        nodes = new HTTPGetProtocol().doQuery(url, getMessageCapabilities(), 0);
        
      return nodes;
    } 

/**
 * It creates the name-value pairs for the getCapabilities request
 * 
 * 
 * @return Name-Value pair array
 */
    public NameValuePair[] getMessageCapabilities() {        
         NameValuePair nvp1 = new NameValuePair("wsdl", "");
         return new NameValuePair[] { nvp1 };
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.IGazetteerServiceDriver#getFeature(java.net.URL, es.gva.cit.gazetteer.querys.Query)
     */

/**
 * 
 * 
 * 
 * @return 
 * @param url 
 * @param query 
 */
    public Collection getFeature(URL url, Query query) {        
        Collection nodes = new java.util.ArrayList();
        setQuery(query);
        
              
       System.out.println("*************POST*************");
       System.out.println(getPOSTGetFeature(query));
       nodes = new HTTPPostProtocol().doQuery(url,getPOSTGetFeature(query),0);
       return nodes;
        
    } 

/**
 * It creates the XML for the getFeature request
 * @return Name-value pair with a XML request
 * @param query 
 */
    public String getPOSTGetFeature(Query query) {        
    	String name = query.getName();
    	if (query.getOptions().getSearch().isWithAccents()){
    		name = Strings.removeAccents(name);
    	}    	
    		return "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" +
        "<SOAP-ENV:Envelope SOAP-ENV:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:SOAP-ENC=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:si=\"http://soapinterop.org/xsd\" xmlns:tns=\"urn:idecwsdl\">" +
       	"<SOAP-ENV:Body><tns:getCoordenadesUTM xmlns:tns=\"urn:idecwsdl\">" +
        "<key xsi:type=\"xsd:string\">CV81HP6</key>" +
        "<toponim xsi:type=\"xsd:string\">" + name + "</toponim>" +
        "</tns:getCoordenadesUTM>" +
        "</SOAP-ENV:Body>" +
        "</SOAP-ENV:Envelope>";
    		
    } 
    
    
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.IGazetteerServiceDriver#isProtocolSupported(java.net.URL)
     */

/**
 * 
 * 
 * 
 * @return 
 * @param url 
 */
    public boolean isProtocolSupported(URL url) {        
        // TODO Auto-generated method stub
        return true;
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.IGazetteerServiceDriver#describeFeatureType(java.net.URL, java.lang.String)
     */

/**
 * 
 * 
 * 
 * @return 
 * @param url 
 * @param feature 
 */
    public Collection describeFeatureType(URL url, String feature) {        
        // TODO Auto-generated method stub
        return null;
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.IGazetteerServiceDriver#setParameters(org.w3c.dom.Node[])
     */

/**
 * 
 * 
 * 
 * @return 
 * @param nodes 
 */
    public boolean setParameters(Collection nodes) {        
        return new IdecCapabilitiesParser(this).parse((XMLNode)nodes.toArray()[0]);
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.IGazetteerServiceDriver#parseFeatures(org.w3c.dom.Node, es.gva.cit.gazetteer.querys.ThesaurusName, java.lang.String)
     */

/**
 * 
 * 
 * 
 * @return 
 * @param node 
 * @param currentThesaurus 
 * @param attribute 
 */
    public Feature[] parseFeatures(XMLNode node, ThesaurusName currentThesaurus, String attribute) {        
        return new IdecFeatureParser().parse(node);
    } 
 }
