/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.filter;

import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.DefaultListModel;
import javax.swing.JPanel;

import org.apache.log4j.Logger;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.instruction.IncompatibleTypesException;
import com.hardcode.gdbms.engine.values.BooleanValue;
import com.hardcode.gdbms.engine.values.DateValue;
import com.hardcode.gdbms.engine.values.NullValue;
import com.hardcode.gdbms.engine.values.StringValue;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.IWindowListener;
import com.iver.andami.ui.mdiManager.WindowInfo;
import com.iver.cit.gvsig.project.documents.table.gui.Table;
import com.iver.utiles.DefaultCharSet;
import com.iver.utiles.StringUtilities;
import com.iver.utiles.exceptionHandling.ExceptionHandlingSupport;
import com.iver.utiles.exceptionHandling.ExceptionListener;
import com.lamatek.swingextras.JDateChooser;


/**
 * DOCUMENT ME!
 *
 * @author Fernando Gonzlez Corts
 */
public class FilterDialog extends JPanel implements IWindow, IWindowListener {
	private static Logger logger = Logger.getLogger(Table.class.getName());
	private ArrayList expressionListeners = new ArrayList();
	private int position = 0;
	private DefaultListModel fieldListModel = new DefaultListModel();
	private DefaultListModel valuesListModel = new DefaultListModel();
	private NumberFormat nf = NumberFormat.getNumberInstance();
	private DateFormat df = DateFormat.getDateInstance();
	private ArrayList variables = new ArrayList();
	private ExpressionDataSource model = null;
	private javax.swing.JPanel jPanel = null;
	private javax.swing.JPanel jPanel1 = null;
	private javax.swing.JPanel jPanel2 = null;
	private javax.swing.JPanel jPanel3 = null;
	private javax.swing.JPanel jPanel4 = null;
	private javax.swing.JLabel jLabel = null;
	private javax.swing.JScrollPane jScrollPane = null;
	private javax.swing.JList lstCampos = null;
	private javax.swing.JButton btnIgual = null;
	private javax.swing.JButton btnMayor = null;
	private javax.swing.JButton btnNot = null;
	private javax.swing.JButton btnDistinto = null;
	private javax.swing.JButton btnMenor = null;
	private javax.swing.JButton btnMayorIgual = null;
	private javax.swing.JButton btnMenorIgual = null;
	private javax.swing.JButton btnAnd = null;
	private javax.swing.JButton btnOr = null;
	private javax.swing.JTextArea txtExpression = null;
	private javax.swing.JPanel jPanel5 = null;
	private javax.swing.JButton btnNuevo = null;
	private javax.swing.JButton btnAdd = null;
	private javax.swing.JButton btnName = null;
	private javax.swing.JButton btnFromSet = null;
	private javax.swing.JScrollPane jScrollPane1 = null;
	private javax.swing.JLabel jLabel1 = null;
	private javax.swing.JList lstValores = null;
	private javax.swing.JScrollPane jScrollPane2 = null;
	private javax.swing.JButton btnParentesis = null;
	private ExceptionHandlingSupport exceptionHandlingSupport = new ExceptionHandlingSupport();
	
	private String title;

	/**
	 * This is the default constructor
	 */
	public FilterDialog(String _title) {		
		super();
		initialize();
		title = _title;
	}
	/**
	 * This is the default constructor
	 */
	public FilterDialog() {		
		super();
		initialize();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param t DOCUMENT ME!
	 */
	public void setModel(ExpressionDataSource t) {
		try {
			model = t;
            model.start();
        } catch (DriverException e1) {
            NotificationManager.addError(e1.getMessage(), e1);
        }
		fieldListModel.clear();

		try {
			for (int i = 0; i < model.getFieldCount(); i++) {
				fieldListModel.addElement(model.getFieldName(i));
			}
		} catch (FilterException e) {
			throwException(e);
		}
	}

	/**
	 * Rellena la lista con los valores del campo seleccionado
	 */
	private void fillValues() {
		int index = lstCampos.getSelectedIndex();

		//Index es ahora el ndice del campo seleccionado
		//Se eliminan los duplicados
		TreeSet conjunto = new TreeSet(new Comparator() {
			public int compare(Object o1, Object o2) {
				if ((o1 != null) && (o2 != null)) {
					Value v2 = (Value) o2;
					Value v1 = (Value) o1;
					BooleanValue boolVal;
					
					try {
						boolVal = (BooleanValue) (v1.greater(v2));
						
						if (boolVal.getValue()) {
							return 1;
						}
						
						boolVal = (BooleanValue) (v1.less(v2));
						
						if (boolVal.getValue()) {
							return -1;
						}
					} catch (IncompatibleTypesException e) {
						throw new RuntimeException(e);
					}
				}
				
				return 0;
			}
		}); // Para poder ordenar
		
		valuesListModel.clear();
		try {
			for (int i = 0; i < model.getRowCount(); i++) {
				Value value = model.getFieldValue(i, index);

				if (value instanceof NullValue)
				    continue;
				
				if (!conjunto.contains(value)) {
				    conjunto.add(value);
				}
			}

			Iterator it = conjunto.iterator();

			while (it.hasNext())
				valuesListModel.addElement(it.next());
		} catch (FilterException e) {
			throwException(e);
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param expresion DOCUMENT ME!
	 * @param substring DOCUMENT ME!
	 * @param startingPos DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	private int getIndex(String expresion, String substring, int startingPos) {
		int index = startingPos;

		do {
			index = expresion.indexOf(substring, index);
		} while ((StringUtilities.isBetweenSymbols(expresion, index, "\"")) &&
				(index != -1));

		return index;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param expresion DOCUMENT ME!
	 * @param word DOCUMENT ME!
	 * @param translation DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws ParseException DOCUMENT ME!
	 */
	private String translateWord(String expresion, String word,
		String translation) throws ParseException {
		int booleanIndex = 0;
		int endIndex = 0;
		StringBuffer res = new StringBuffer();

		while ((booleanIndex = getIndex(expresion, word, booleanIndex)) != -1) {
			res.append(expresion.substring(endIndex, booleanIndex));
			endIndex = booleanIndex + word.length();
			booleanIndex++;
			res.append(translation);
		}

		if (endIndex < expresion.length()) {
			res.append(expresion.substring(endIndex));
		}

		return res.toString();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param expresion DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws ParseException DOCUMENT ME!
	 */
	private String translateDates(String expresion) throws ParseException {
		//Se obtiene el valor de la fecha
		String date = StringUtilities.substringDelimited(expresion, "Date(",
				")", 0);

		if (date == null) {
			return expresion;
		}

		//Se comprueba que no est entre comillas 
		int startIndex = expresion.indexOf(date);

		while (startIndex != -1) {
			if (!StringUtilities.isBetweenSymbols(expresion, startIndex, "\"")) {
				//Se sustituye por el valor ordinal de la fecha
				expresion = expresion.substring(0, startIndex - 5) +
					expresion.substring(startIndex).replaceFirst(date + "\\)",
						new Long((df.parse(date)).getTime()).toString());
				;
			} else {
				startIndex += date.length();
			}

			//Se obtiene el valor de la fecha

			/*            date = StringUtilities.substringDelimited(expresion, "Date(", ")",
			   startIndex);
			 */
			if (date == null) {
				return expresion;
			}

			startIndex = expresion.indexOf(date, startIndex);
		}

		return expresion;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param expresion DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws ParseException DOCUMENT ME!
	 */
	public String translateNumber(String expresion) throws ParseException {
		DefaultCharSet ss = new DefaultCharSet();
		ss.addInterval('0', '9');
		ss.addCharacter(',');
		ss.addCharacter('.');

		String number = StringUtilities.substringWithSymbols(expresion, ss, 0);

		if (number == null) {
			return expresion;
		}

		int startIndex = expresion.indexOf(number);

		while (startIndex != -1) {
			Number n = nf.parse(number);

			if (!StringUtilities.isBetweenSymbols(expresion, startIndex, "\"")) {
				//Se sustituye por el valor ordinal de la fecha
				expresion = expresion.substring(0, startIndex) +
					expresion.substring(startIndex).replaceFirst(number,
						n.toString());
			} else {
				startIndex += n.toString().length();
			}

			number = StringUtilities.substringWithSymbols(expresion, ss,
					startIndex);

			if (number == null) {
				return expresion;
			}

			startIndex = expresion.indexOf(number, startIndex);
		}

		return expresion;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws ParseException DOCUMENT ME!
	 */
	private String validateExpression() throws ParseException {
		String expression = txtExpression.getText();
		HashSet variablesIndexes = new HashSet();

		StringBuffer traducida = new StringBuffer();

		//Se transforman los nombres de los campos en las variables xix que analizarn
		//Se quitan los Date(fecha) y se mete la fecha correspondiente
		expression = translateDates(expression);
		expression = translateNumber(expression);
		expression = translateWord(expression, "true", "1");
		expression = translateWord(expression, "false", "0");

		String replacement;
		Pattern patron = Pattern.compile("[^<>!]=");
		Matcher m = patron.matcher(expression);
		int index = 0;

		while (m.find(index)) {
			index = m.start();
			replacement = expression.charAt(index) + "==";
			m.replaceFirst(replacement);
			index++;
		}

		expression = expression.replaceAll("[^<>!]=", "==");

		logger.debug(expression);

		return expression;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param symbol DOCUMENT ME!
	 */
	private void putSymbol(String symbol) {
		int position = txtExpression.getCaretPosition();
		txtExpression.setText(StringUtilities.insert(txtExpression.getText(),
				position, symbol));

		if (symbol.equals(" () ")) {
			position = position + 2;
		} else {
			position = position + symbol.length();
		}

		txtExpression.setCaretPosition(position);
	}

	/**
	 * This method initializes this
	 */
	private void initialize() {
		this.setLayout(new java.awt.BorderLayout());
		this.add(getJPanel(), java.awt.BorderLayout.CENTER);
		this.add(getJPanel1(), java.awt.BorderLayout.SOUTH);
	}

	/**
	 * This method initializes jPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new javax.swing.JPanel();
			jPanel.setLayout(new java.awt.BorderLayout());
			jPanel.add(getJPanel2(), java.awt.BorderLayout.WEST);
			jPanel.add(getJPanel3(), java.awt.BorderLayout.CENTER);
			jPanel.add(getJPanel4(), java.awt.BorderLayout.EAST);
		}

		return jPanel;
	}

	/**
	 * This method initializes jPanel1
	 *
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanel1() {
		if (jPanel1 == null) {
			jPanel1 = new javax.swing.JPanel();
			jPanel1.add(getJScrollPane2(), null);
			jPanel1.add(getJPanel5(), null);
			jPanel1.setPreferredSize(new java.awt.Dimension(10, 110));
		}

		return jPanel1;
	}

	/**
	 * This method initializes jPanel2
	 *
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanel2() {
		if (jPanel2 == null) {
			jPanel2 = new javax.swing.JPanel();
			jPanel2.add(getJLabel(), null);
			jPanel2.add(getJScrollPane(), null);
			jPanel2.setPreferredSize(new java.awt.Dimension(150, 170));
		}

		return jPanel2;
	}

	/**
	 * This method initializes jPanel3
	 *
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanel3() {
		if (jPanel3 == null) {
			jPanel3 = new javax.swing.JPanel();
			jPanel3.add(getBtnIgual(), null);
			jPanel3.add(getBtnDistinto(), null);
			jPanel3.add(getBtnMayor(), null);
			jPanel3.add(getBtnMenor(), null);
			jPanel3.add(getBtnMenorIgual(), null);
			jPanel3.add(getBtnMayorIgual(), null);
			jPanel3.add(getBtnAnd(), null);
			jPanel3.add(getBtnOr(), null);
			jPanel3.add(getBtnNot(), null);
			jPanel3.add(getBtnParentesis(), null);
			jPanel3.add(getBtnName(), null);
			jPanel3.setPreferredSize(new java.awt.Dimension(180, 36));
		}

		return jPanel3;
	}

	/**
	 * This method initializes jPanel4
	 *
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanel4() {
		if (jPanel4 == null) {
			jPanel4 = new javax.swing.JPanel();
			jPanel4.add(getJLabel1(), null);
			jPanel4.add(getJScrollPane1(), null);
			jPanel4.setPreferredSize(new java.awt.Dimension(150, 10));
		}

		return jPanel4;
	}

	/**
	 * This method initializes jLabel
	 *
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabel() {
		if (jLabel == null) {
			jLabel = new javax.swing.JLabel();
			jLabel.setText(PluginServices.getText(this, "Campos") + ":");
		}

		return jLabel;
	}

	/**
	 * This method initializes jScrollPane
	 *
	 * @return javax.swing.JScrollPane
	 */
	private javax.swing.JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new javax.swing.JScrollPane();
			jScrollPane.setViewportView(getLstCampos());
			jScrollPane.setPreferredSize(new java.awt.Dimension(135, 120));
		}

		return jScrollPane;
	}

	/**
	 * This method initializes lstCampos
	 *
	 * @return javax.swing.JList
	 */
	private javax.swing.JList getLstCampos() {
		if (lstCampos == null) {
			lstCampos = new javax.swing.JList();
			lstCampos.setModel(fieldListModel);
			lstCampos.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
			lstCampos.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
					public void valueChanged(
						javax.swing.event.ListSelectionEvent e) {
						PluginServices.backgroundExecution(new Runnable() {
							public void run() {
								fillValues();
							}
						});
					}
				});
			lstCampos.addMouseListener(new java.awt.event.MouseAdapter() {
					public void mouseClicked(java.awt.event.MouseEvent e) {
						if (e.getClickCount() == 2) {
							putSymbol((String) fieldListModel.getElementAt(
									getLstCampos().getSelectedIndex()));
						}
					}
				});
		}

		return lstCampos;
	}

	/**
	 * This method initializes btnIgual
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnIgual() {
		if (btnIgual == null) {
			btnIgual = new javax.swing.JButton();
			btnIgual.setText("=");
			btnIgual.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnIgual.setPreferredSize(new java.awt.Dimension(40, 20));
			btnIgual.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						putSymbol(" = ");
					}
				});
		}

		return btnIgual;
	}

	/**
	 * This method initializes btnMayor
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnMayor() {
		if (btnMayor == null) {
			btnMayor = new javax.swing.JButton();
			btnMayor.setText(">");
			btnMayor.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnMayor.setPreferredSize(new java.awt.Dimension(40, 20));
			btnMayor.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						putSymbol(" > ");
					}
				});
		}

		return btnMayor;
	}

	/**
	 * This method initializes btnNot
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnNot() {
		if (btnNot == null) {
			btnNot = new javax.swing.JButton();
			btnNot.setText("Not");
			btnNot.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnNot.setPreferredSize(new java.awt.Dimension(40, 20));
			btnNot.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						putSymbol(" not ");
					}
				});
		}

		return btnNot;
	}

	/**
	 * This method initializes btnDistinto
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnDistinto() {
		if (btnDistinto == null) {
			btnDistinto = new javax.swing.JButton();
			btnDistinto.setText("!=");
			btnDistinto.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnDistinto.setPreferredSize(new java.awt.Dimension(40, 20));
			btnDistinto.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						putSymbol(" <> ");
					}
				});
		}

		return btnDistinto;
	}

	/**
	 * This method initializes btnMenor
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnMenor() {
		if (btnMenor == null) {
			btnMenor = new javax.swing.JButton();
			btnMenor.setText("<");
			btnMenor.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnMenor.setPreferredSize(new java.awt.Dimension(40, 20));
			btnMenor.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						putSymbol(" < ");
					}
				});
		}

		return btnMenor;
	}

	/**
	 * This method initializes btnMayorIgual
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnMayorIgual() {
		if (btnMayorIgual == null) {
			btnMayorIgual = new javax.swing.JButton();
			btnMayorIgual.setText(">=");
			btnMayorIgual.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnMayorIgual.setPreferredSize(new java.awt.Dimension(40, 20));
			btnMayorIgual.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						putSymbol(" >= ");
					}
				});
		}

		return btnMayorIgual;
	}

	/**
	 * This method initializes btnMenorIgual
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnMenorIgual() {
		if (btnMenorIgual == null) {
			btnMenorIgual = new javax.swing.JButton();
			btnMenorIgual.setText("<=");
			btnMenorIgual.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnMenorIgual.setPreferredSize(new java.awt.Dimension(40, 20));
			btnMenorIgual.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						putSymbol(" <= ");
					}
				});
		}

		return btnMenorIgual;
	}

	/**
	 * This method initializes btnAnd
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnAnd() {
		if (btnAnd == null) {
			btnAnd = new javax.swing.JButton();
			btnAnd.setText("And");
			btnAnd.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnAnd.setPreferredSize(new java.awt.Dimension(40, 20));
			btnAnd.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						putSymbol(" and ");
					}
				});
		}

		return btnAnd;
	}

	/**
	 * This method initializes btnOr
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnOr() {
		if (btnOr == null) {
			btnOr = new javax.swing.JButton();
			btnOr.setText("Or");
			btnOr.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnOr.setPreferredSize(new java.awt.Dimension(40, 20));
			btnOr.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						putSymbol(" or ");
					}
				});
		}

		return btnOr;
	}

	/**
	 * This method initializes txtExpression
	 *
	 * @return javax.swing.JTextArea
	 */
	private javax.swing.JTextArea getTxtExpression() {
		if (txtExpression == null) {
			txtExpression = new javax.swing.JTextArea();
			txtExpression.setLineWrap(true);
		}

		return txtExpression;
	}

	/**
	 * This method initializes jPanel5
	 *
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanel5() {
		if (jPanel5 == null) {
			jPanel5 = new javax.swing.JPanel();
			jPanel5.add(getBtnNuevo(), null);
			jPanel5.add(getBtnAdd(), null);
			jPanel5.add(getBtnFromSet(), null);
			jPanel5.setPreferredSize(new java.awt.Dimension(190, 100));
		}

		return jPanel5;
	}

	/**
	 * This method initializes btnNuevo
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnNuevo() {
		if (btnNuevo == null) {
			btnNuevo = new javax.swing.JButton();
			btnNuevo.setText(PluginServices.getText(this, "Nuevo_conjunto"));
			btnNuevo.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnNuevo.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						final String expr = "select * from '" +
							model.getDataSourceName() + "' where " +
							getTxtExpression().getText() + ";";

						logger.debug(expr);

						PluginServices.backgroundExecution(new Runnable() {
							public void run() {
								for (int i = 0; i < expressionListeners.size(); i++) {
									ExpressionListener l = (ExpressionListener) expressionListeners.get(i);
									l.newSet(expr);
								}
							}
						});
					}
				});
		}

		return btnNuevo;
	}

	/**
	 * This method initializes btnAdd
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnAdd() {
		if (btnAdd == null) {
			btnAdd = new javax.swing.JButton();
			btnAdd.setText(PluginServices.getText(this, "Anadir_al_conjunto"));
			btnAdd.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnAdd.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						final String expr = "select * from '" +
							model.getDataSourceName() + "' where " +
							getTxtExpression().getText() + ";";

						logger.debug(expr);

						PluginServices.backgroundExecution(new Runnable() {
								public void run() {
									for (int i = 0;
											i < expressionListeners.size();
											i++) {
										ExpressionListener l = (ExpressionListener) expressionListeners.get(i);
										l.addToSet(expr);
									}
								}
							});
					}
				});
		}

		return btnAdd;
	}

	/**
	 * This method initializes btnFromSet
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnFromSet() {
		if (btnFromSet == null) {
			btnFromSet = new javax.swing.JButton();
			btnFromSet.setText(PluginServices.getText(this,
					"Seleccionar_del_conjunto"));
			btnFromSet.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnFromSet.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						final String expr = "select * from '" +
							model.getDataSourceName() + "' where " +
							getTxtExpression().getText() + ";";

						logger.debug(expr);

						PluginServices.backgroundExecution(new Runnable() {
							public void run() {
								for (int i = 0; i < expressionListeners.size(); i++) {
									ExpressionListener l = (ExpressionListener) expressionListeners.get(i);
									l.fromSet(expr);
								}
							}
						});
					}
				});
		}

		return btnFromSet;
	}

	/**
	 * This method initializes jScrollPane1
	 *
	 * @return javax.swing.JScrollPane
	 */
	private javax.swing.JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new javax.swing.JScrollPane();
			jScrollPane1.setViewportView(getLstValores());
			jScrollPane1.setPreferredSize(new java.awt.Dimension(140, 120));
		}

		return jScrollPane1;
	}

	/**
	 * This method initializes jLabel1
	 *
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabel1() {
		if (jLabel1 == null) {
			jLabel1 = new javax.swing.JLabel();
			jLabel1.setText(PluginServices.getText(this, "Valores") + ":");
		}

		return jLabel1;
	}

	/**
	 * This method initializes lstValores
	 *
	 * @return javax.swing.JList
	 */
	private javax.swing.JList getLstValores() {
		if (lstValores == null) {
			lstValores = new javax.swing.JList();
			lstValores.setModel(valuesListModel);
			lstValores.addMouseListener(new java.awt.event.MouseAdapter() {
					public void mouseClicked(java.awt.event.MouseEvent e) {
						if (e.getClickCount() == 2) {
							Value valor = (Value) valuesListModel.getElementAt(getLstValores()
																				   .getSelectedIndex());

							if (valor instanceof DateValue) {
								putSymbol("date('" + valor + "')");
							} else if (valor instanceof BooleanValue) {
								putSymbol("boolean('" + valor.toString() +
									"')");
							} else if (valor instanceof StringValue) {
								putSymbol("'" + valor + "'");
							} else {
								putSymbol(valor.toString());
							}
						}
					}
				});
		}

		return lstValores;
	}

	/**
	 * This method initializes jScrollPane2
	 *
	 * @return javax.swing.JScrollPane
	 */
	private javax.swing.JScrollPane getJScrollPane2() {
		if (jScrollPane2 == null) {
			jScrollPane2 = new javax.swing.JScrollPane();
			jScrollPane2.setViewportView(getTxtExpression());
			jScrollPane2.setPreferredSize(new java.awt.Dimension(250, 100));
			jScrollPane2.setHorizontalScrollBarPolicy(javax.swing.JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		}

		return jScrollPane2;
	}

	/**
	 * This method initializes btnParentesis
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnParentesis() {
		if (btnParentesis == null) {
			btnParentesis = new javax.swing.JButton();
			btnParentesis.setText("()");
			btnParentesis.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnParentesis.setPreferredSize(new java.awt.Dimension(40, 20));
			btnParentesis.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						putSymbol(" () ");
					}
				});
		}

		return btnParentesis;
	}

	/**
	 * This method initializes btnName
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnName() {
		if (btnName == null) {
			btnName = new javax.swing.JButton();
			btnName.setText("Date");
			btnName.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						JDateChooser chooser = new JDateChooser();

						PluginServices.getMDIManager().addWindow(chooser);

						if (chooser.getSelectedDate() != null){
							Date selectedDate = chooser.getSelectedDate().getTime();

							if (selectedDate != null) {
								putSymbol("Date(" + df.format(selectedDate) + ")");
							}
						}
					}
				});
		}

		return btnName;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param arg0
	 *
	 * @return
	 */
	public boolean addExpressionListener(ExpressionListener arg0) {
		return expressionListeners.add(arg0);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param arg0
	 *
	 * @return
	 */
	public boolean removeExpressionListener(ExpressionListener arg0) {
		return expressionListeners.remove(arg0);
	}

	/**
	 * @see com.iver.mdiApp.ui.MDIManager.IWindow#getWindowInfo()
	 */
	public WindowInfo getWindowInfo() {
		WindowInfo vi = new WindowInfo(WindowInfo.ICONIFIABLE);
		vi.setWidth(480);
		vi.setHeight(362);
		vi.setTitle(PluginServices.getText( this, "filtro") + " (" + title + ")");
		return vi;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param o DOCUMENT ME!
	 */
	public void addExceptionListener(ExceptionListener o) {
		exceptionHandlingSupport.addExceptionListener(o);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param o DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public boolean removeExceptionListener(ExceptionListener o) {
		return exceptionHandlingSupport.removeExceptionListener(o);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param t DOCUMENT ME!
	 */
	private void throwException(Throwable t) {
		exceptionHandlingSupport.throwException(t);
	}

    /* (non-Javadoc)
     * @see com.iver.andami.ui.mdiManager.ViewListener#viewActivated()
     */
    public void windowActivated() {
    }

    /* (non-Javadoc)
     * @see com.iver.andami.ui.mdiManager.ViewListener#viewClosed()
     */
    public void windowClosed() {
        try {
            model.stop();
        } catch (DriverException e) {
            NotificationManager.addError(e.getMessage(), e);
        }        
    }
}

//  @jve:visual-info  decl-index=0 visual-constraint="10,10"
