/*
 * Created on 27-abr-2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.project.documents.view.legend.gui;

import java.awt.Dimension;
import java.awt.GridLayout;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import com.hardcode.gdbms.engine.values.NullValue;
import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.core.ISymbol;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.rendering.NullIntervalValue;
import com.iver.cit.gvsig.fmap.rendering.NullUniqueValue;
import com.iver.cit.gvsig.project.documents.view.legend.edition.gui.FCellSymbolRenderer;
import com.iver.cit.gvsig.project.documents.view.legend.edition.gui.IntervalCellEditor;
import com.iver.cit.gvsig.project.documents.view.legend.edition.gui.SymbolCellEditor;
import com.iver.cit.gvsig.project.documents.view.legend.edition.gui.ValueCellEditor;
import com.iver.utiles.swing.jtable.JTable;
import com.iver.utiles.swing.jtable.TextFieldCellEditor;


/**
 * JPanel que contiene la tabla con los smbolos intervalos o valores y
 * etiquetado de estos valores.
 *
 * @author Vicente Caballero Navarro
 */
public class FSymbolTable extends JPanel {
    private static final long serialVersionUID = 1L;

    //	private boolean DEBUG = true;
    // private  MyTableModel m_TableModel;
    private JTable table;
    private String type;

    /**
     * Crea un nuevo FSymbolTable.
     *
     * @param type tipo de valor si es intervalo: "intervals" y si es por
     *        valores: "values".
     */
    public FSymbolTable(String type) {
        super(new GridLayout(1, 0));
        this.type = type;
        table = new JTable();
        table.setModel(new MyTableModel());
        table.setPreferredScrollableViewportSize(new Dimension(480, 110));

        //Create the scroll pane and add the table to it.
        JScrollPane scrollPane = new JScrollPane(table);

        //Set up column sizes.
        //initColumnSizes(table);
        setUpSymbolColumn(table, table.getColumnModel().getColumn(0));
        setUpValueColumn(table, table.getColumnModel().getColumn(1));
        setUpLabelColumn(table, table.getColumnModel().getColumn(2));

        //Add the scroll pane to this panel.
        add(scrollPane);
        table.setRowSelectionAllowed(true);
    }

    /**
     * Aade una fila al modelo.
     *
     * @param vector Fila en forma de vector de Object para aadir al modelo.
     */
    public void addRow(Object[] vector) {
        MyTableModel m = (MyTableModel) table.getModel();
        m.addRow(vector);
    }

    /**
     * Elimina la fila que tiene como clave el objeto que se pasa como
     * parmetro.
     *
     * @param obj clave del objeto a eliminar.
     */
    public void removeRow(Object obj) {
        MyTableModel m = (MyTableModel) table.getModel();

        for (int i = 0; i < m.getRowCount(); i++) {
            if (m.getValueAt(i, 1) instanceof NullUniqueValue ||
                    m.getValueAt(i, 1) instanceof NullIntervalValue) {
                m.removeRow(i);
            }
        }
    }

    /**
     * Elimina las filas que estn seleccionadas.
     */
    public void removeSelectedRows() {
        if (table.getCellEditor() != null) {
            table.getCellEditor().cancelCellEditing();
        }

        MyTableModel m = (MyTableModel) table.getModel();
        int[] selectedRows = table.getSelectedRows();

        for (int i = selectedRows.length - 1; i >= 0; i--) {
            m.removeRow(selectedRows[i]);
        }
    }

    /**
     * Rellena la tabla con los smbolos valores y descripciones que se pasan
     * como parmetro.
     *
     * @param symbols Array de smbolos
     * @param values Array de valores.
     * @param descriptions Array de descripciones.
     */
    public void fillTableFromSymbolList(ISymbol[] symbols, Object[] values,
        String[] descriptions) {
        FSymbol theSymbol;

        for (int i = 0; i < symbols.length; i++) {
            theSymbol = (FSymbol) symbols[i];
            addTableRecord(theSymbol, values[i], descriptions[i]);
        }
    }

    /**
     * Aade una fila con los objetos que se pasan como parmetros.
     *
     * @param symbol smbolo de la fila.
     * @param value Valor de la fila.
     * @param description Descripcin.
     */
    public void addTableRecord(ISymbol symbol, Object value, String description) {
        Object[] theRow = new Object[3];
        theRow[0] = symbol;
        theRow[1] = value;
        theRow[2] = description;
        addRow(theRow);
    }

    /**
     * Devuelve el valor a partie del nmero de fila y columna.
     *
     * @param row nmero de fila.
     * @param col nmero de columna.
     *
     * @return Objeto.
     */
    public Object getFieldValue(int row, int col) {
        MyTableModel m = (MyTableModel) table.getModel();

        return m.getValueAt(row, col);
    }

    /**
     * Devuelve el nmero total de filas que contiene el modelo.
     *
     * @return Nmero de filas.
     */
    public int getRowCount() {
        MyTableModel m = (MyTableModel) table.getModel();

        return m.getRowCount();
    }

    /**
     * Elimina todas las filas del modelo.
     */
    public void removeAllItems() {
        table.setModel(new MyTableModel());
        setUpSymbolColumn(table, table.getColumnModel().getColumn(0));
        setUpValueColumn(table, table.getColumnModel().getColumn(1));
        setUpLabelColumn(table, table.getColumnModel().getColumn(2));
    }

    /**
     * Inicializa el cell editor de tipo descripcin de la columna que se pasa
     * como parmetro.
     *
     * @param table2 Tabla.
     * @param column Columna.
     */
    public void setUpLabelColumn(JTable table2, TableColumn column) {
        TextFieldCellEditor labeleditor = new TextFieldCellEditor();
        column.setCellEditor(labeleditor);
    }

    /**
     * Inicializa el cell editor de tipo valor de la columna que se pasa como
     * parmetro.
     *
     * @param table2 Tabla.
     * @param column Columna.
     */
    public void setUpValueColumn(JTable table2, TableColumn column) {
        if (type.equals("intervals")) {
            //FIntervalCellEditor intervaleditor = new FIntervalCellEditor();
            IntervalCellEditor intervaleditor = new IntervalCellEditor();
            column.setCellEditor(intervaleditor);

            ///FCellIntervalRenderer renderer = new FCellIntervalRenderer(true);
            ///column.setCellRenderer(renderer);
        } else {
            ///FValueCellEditor valueeditor = new FValueCellEditor();
            ///TextFieldCellEditor valueeditor = new TextFieldCellEditor();
            ValueCellEditor valueeditor = new ValueCellEditor();
            column.setCellEditor(valueeditor);
        }
    }

    /*
     * This method picks good column sizes.
     * If all column heads are wider than the column's cells'
     * contents, then you can just use column.sizeWidthToFit().
     */

    //private void initColumnSizes(JTable table) {
    //MyTableModel model = (MyTableModel) table.getModel();
    //TableColumn column = null;
    //Component comp = null;
    //int headerWidth = 0;
    //int cellWidth = 0;
    //TableCellRenderer headerRenderer = table.getTableHeader()
    //										.getDefaultRenderer();
    //}

    /**
     * Inicializa el cell editor de tipo smbolo de la columna que se pasa como
     * parmetro.
     *
     * @param table2 Tabla.
     * @param column Columna.
     */
    public void setUpSymbolColumn(JTable table2, TableColumn column) {
        //Set up the editor 
        column.setMaxWidth(100);
        column.setWidth(60);
        column.setPreferredWidth(60);
        column.setMinWidth(50);

        //FSymbolCellEditor symboleditor = new FSymbolCellEditor();
        SymbolCellEditor symboleditor = new SymbolCellEditor();
        column.setCellEditor(symboleditor);

        FCellSymbolRenderer renderer = new FCellSymbolRenderer(true);
        column.setCellRenderer(renderer);
    }

    /**
     * Modelo que propio que se aplica a la tabla.
     *
     * @author Vicente Caballero Navarro
     */
    class MyTableModel extends DefaultTableModel { /**
         *
         */

        private static final long serialVersionUID = 1L;

        //  AbstractTableModel {
        private String[] columnNames = {
                PluginServices.getText(this, "Simbolo"),
                PluginServices.getText(this, "Valor"),
                PluginServices.getText(this, "Etiqueta")
            };

        /**
         * Devuelve el nmero de columnas.
         *
         * @return Nmero de columnas.
         */
        public int getColumnCount() {
            return columnNames.length;
        }

        /**
         * Devuelve el String del valor de la columna.
         *
         * @param col Nmero de columna.
         *
         * @return Nombre de la columna.
         */
        public String getColumnName(int col) {
            return columnNames[col];
        }

        /*
         * JTable uses this method to determine the default renderer/
         * editor for each cell.  If we didn't implement this method,
         * then the last column would contain text ("true"/"false"),
         * rather than a check box.
         */
        public Class getColumnClass(int c) {
            if (getValueAt(0, c) == null) {
                return NullValue.class;
            }

            return getValueAt(0, c).getClass();
        }

        /*
         * Don't need to implement this method unless your table's
         * editable.
         */
        public boolean isCellEditable(int row, int col) {
            //Note that the data/cell address is constant,
            //no matter where the cell appears onscreen.
            //if (col > 0) {
            return true;

            /* } else {
               return false;
               } */
        }

        /*
         * Don't need to implement this method unless your table's
         * data can change.
         */
        /* public void setValueAt(Object value, int row, int col) {
           if (DEBUG) {
                   System.out.println("Setting value at " + row + "," + col
                                                      + " to " + value
                                                      + " (an instance of "
                                                      + value.getClass() + ")");
           }
           // FSymbol theSymbol;
           switch (col)
           {
                   case 0: // Simbolo
                           // m_FRenderer.setValueSymbol(row,(FSymbol) value);
                           break;
                   case 1: // Clave
                           FInterval newInterval = FInterval.parseString((String) value);
                           this.setValueAt(newInterval,row,col);
                           break;
                   case 2: // Etiqueta
                           // theSymbol = m_FRenderer.getSymbolByID(row);
                           // theSymbol.m_Descrip = (String) value;
        
           }
           // data[row][col] = value;
           fireTableCellUpdated(row, col);
           }  */
        /* private void printDebugData() {
           int numRows = getRowCount();
           int numCols = getColumnCount();
           for (int i=0; i < numRows; i++) {
                   System.out.print("    row " + i + ":");
                   for (int j=0; j < numCols; j++) {
                           System.out.print("  " + data[i][j]);
                   }
                   System.out.println();
           }
           System.out.println("--------------------------");
           } */
    }
}
