/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.wizards;

import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.geom.Rectangle2D;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

import org.cresques.cts.IProjection;
import org.cresques.io.GeoRasterFile;
import org.cresques.px.Extent;
import org.gvsig.georeferencing.gui.dialog.GeoreferencingDialog;
import org.gvsig.georeferencing.utils.GeoPointPersistence;
import org.gvsig.georeferencing.utils.StackZoom;

import com.hardcode.driverManager.DriverLoadException;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.RasterDriver;
import com.iver.cit.gvsig.fmap.layers.FLyrGeoRaster;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.RasterAdapter;
import com.iver.cit.gvsig.fmap.layers.RasterFileAdapter;
import com.iver.cit.gvsig.project.documents.gui.FOpenDialog;
import com.iver.cit.gvsig.project.documents.view.gui.View;
import com.iver.cit.gvsig.gui.panels.CRSSelectPanel;

/**
 * Esta clase gestiona la carga de ficheros a georreferenciar mostrando un
 * wizard para su seleccin. 
 * 
 * NO SE USA!!!!! 
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class SelectFilePanel extends JPanel {

	private JPanel pGeneral = null;
	private JPanel pFileSelection = null;
	private JPanel pControls = null;
	private JTextField tFile = null;
	private JButton bSelectFile = null;
	//private ProjChooserPanel jPanelProj = null;
	private CRSSelectPanel pProyection = null;
	/**
	 * Lista de formatos soportados
	 */
	private String[] fileFilters = {"ecw", "sid", "tif", "jpg", "png"};
	/**
	 * Nombre del fichero seleccionado
	 */
	private String fName = "";
	
	/**
	 * Nombre de la capa
	 */
	private String lyrName = "";
	/**
	 * Recuerda la ltima ruta seleccionada por el usuario
	 */
	private String lastPath = "./";
	private GeoreferencingDialog parent = null;
	private FLyrGeoRaster lyrGeoRaster = null;
	
	private JPanel pCheckUseGeoref = null;
	private JCheckBox cbUseGeoref = null;
	private JLabel lUseGeoref = null;
	public double widthPxImg, heightPxImg;
	
	/**
	 * This is the default constructor
	 */
	public SelectFilePanel(GeoreferencingDialog georef) {
		super();
		initialize();
		this.parent = georef;
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
        this.setPreferredSize(new java.awt.Dimension(400,240));
        this.setSize(new java.awt.Dimension(400,240));
        this.setLocation(new java.awt.Point(0,0));
        this.add(getPGeneral(), null);
        this.getBSelectFile().addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	acceptButtonActionPerformed(evt);
            }
        });
	 }

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPGeneral() {
		if (pGeneral == null) {
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.insets = new java.awt.Insets(5,0,0,0);
			gridBagConstraints1.gridy = 1;
			gridBagConstraints1.gridx = 0;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.insets = new java.awt.Insets(0,0,2,0);
			gridBagConstraints.gridy = 0;
			gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
			gridBagConstraints.gridx = 0;
			pGeneral = new JPanel();
			pGeneral.setLayout(new GridBagLayout());
			pGeneral.setPreferredSize(new java.awt.Dimension(400,240));
			gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTH;
			pGeneral.add(getPFileSelection(), gridBagConstraints);
			pGeneral.add(getPControls(), gridBagConstraints1);
		}
		return pGeneral;
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPFileSelection() {
		if (pFileSelection == null) {
			FlowLayout flowLayout = new FlowLayout();
			flowLayout.setVgap(10);
			pFileSelection = new JPanel();
			pFileSelection.setBorder(javax.swing.BorderFactory.createTitledBorder(null, PluginServices.getText(this,"cargar"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
			pFileSelection.setLayout(flowLayout);
			pFileSelection.setPreferredSize(new java.awt.Dimension(380,70));
			pFileSelection.add(getTFile(), null);
			pFileSelection.add(getBSelectFile(), null);
		}
		return pFileSelection;
	}

	private JPanel getPControls() {
		if (pControls == null) {
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 1;
			pControls = new JPanel();
			pControls.setLayout(new GridBagLayout());
			pControls.setPreferredSize(new java.awt.Dimension(380,135));
			pControls.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
			gridBagConstraints11.gridx = 0;
			gridBagConstraints11.gridy = 0;
			pControls.add(getPCheckUseGeoref(), gridBagConstraints11);
			pControls.add(getPProyection(), gridBagConstraints2);
		}
		return pControls;
	}
	
	/**
	 * This method initializes jTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getTFile() {
		if (tFile == null) {
			tFile = new JTextField();
			tFile.setPreferredSize(new java.awt.Dimension(280,25));
		}
		return tFile;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getBSelectFile() {
		if (bSelectFile == null) {
			bSelectFile = new JButton();
			bSelectFile.setText(PluginServices.getText(this, "cargar"));
		}
		return bSelectFile;
	}
	
	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private CRSSelectPanel getPProyection() {
		if (pProyection == null) {
			pProyection = CRSSelectPanel.getPanel(GeoreferencingDialog.getLastProjection());
			pProyection.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
			        if (pProyection.isOkPressed()) {
			        	FOpenDialog.setLastProjection(pProyection.getCurProj());
			        }
				}
			});
		}
		return pProyection;
	}

	/**
	 * Crea una capa Raster a partir del nombre driver, fichero y proyeccin y 
	 * coordenadas de georeferenciacin. Esta funcin es para georeferenciar 
	 * capas raster. Para imagenes que no tienen georeferenciacin hay que asignarle
	 * una temporal, normalmente a partir de la vista activa.
	 *
	 * @param layerName Nombre de la capa.
	 * @param d RasterDriver.
	 * @param f Fichero.
	 * @param proj Proyeccin.
	 * @param extent Extent de la vista activa
	 *
	 * @return Nueva capa de tipo raster.
	 *
	 * @throws DriverIOException
	 */
	private FLyrGeoRaster createLayer(String layerName, RasterDriver d,
		File f, IProjection proj) throws DriverException {
		RasterAdapter adapter = new RasterFileAdapter(f);
		adapter.setDriver(d);
		GeoPointPersistence	geoPointPersistence = new GeoPointPersistence();
		StackZoom zoom = new StackZoom();
		FLyrGeoRaster capa = new FLyrGeoRaster(geoPointPersistence, zoom);
		geoPointPersistence.setLyrPoints(capa.getFLyrPoints());
		
		if (capa != null) {
			capa.setName(layerName);
			capa.setSource(adapter);
			capa.setProjection(proj);
			try {
				capa.load();
			} catch (DriverIOException e) {
				throw new DriverException(e);
			}
	        
	        capa.setVisible(true);
		}else 
			return null;
		return capa;
	}
	
	/**
	 * Calcula un extent posible para la imagen a partir del extent de la vista.
	 * En este caso centra la imagen en la vista.
	 * @param w	Ancho de la imagen
	 * @param h Alto de la imagen
	 * @return	Extent para la imagen
	 */
	protected Extent calcTempExtent(String file, ViewPort vp, IProjection proj){
		
		//Obtenemos el ancho y alto de la imagen y obtenemos un extent a partir
		//del viewport.
		Extent tempExtent = null;
		GeoRasterFile grf = GeoRasterFile.openFile(proj, file);
		widthPxImg = grf.getWidth();
		heightPxImg = grf.getHeight();
		if(vp == null || vp.getAdjustedExtent() == null){
			vp = new ViewPort(proj);
			Rectangle2D r2d = new Rectangle2D.Double(0, 0, widthPxImg, heightPxImg);
			vp.setExtent(r2d);
			tempExtent = new Extent(0, 0, widthPxImg, heightPxImg);
		}else
			tempExtent = new Extent(vp.getAdjustedExtent());
		grf.close();
		
		
		double ulX = 0D, ulY = 0D, lrX = 0D, lrY = 0D;
		if(widthPxImg > heightPxImg){
			double widthView = tempExtent.maxX() - tempExtent.minX();
			ulX = tempExtent.minX() + (widthView / 4);
			lrX = ulX + (widthView / 2);
			double newAlto = ((heightPxImg * (widthView / 2)) / widthPxImg);
			double centroY = tempExtent.minY()+((tempExtent.maxY() - tempExtent.minY())/2);
			ulY = centroY - (newAlto / 2);
			lrY = centroY + (newAlto / 2);
		}else{
			double heightView = tempExtent.maxY() - tempExtent.minY();
			ulY = tempExtent.minY() + (heightView / 4);
			lrY = ulY + (heightView / 2);
			double newAncho = ((widthPxImg * (heightView / 2)) / heightPxImg);
			double centroX = tempExtent.minX()+((tempExtent.maxX() - tempExtent.minX())/2);
			ulX = centroX - (newAncho / 2);
			lrX = centroX + (newAncho / 2);
		}
		return new Extent(ulX, ulY, lrX, lrY);
	}
	
	/**
	 * Evento de pulsado del botn de seleccionar fichero
	 * @param e
	 */
	private void acceptButtonActionPerformed(ActionEvent e) {
		//Selector de Fichero que se quiere georeferenciar
		
		if(	e.getSource().equals(this.getBSelectFile())){
			JFileChooser chooser = new JFileChooser(lastPath);
			chooser.setDialogTitle(PluginServices.getText(this, "seleccionar_fichero"));
			FileFilter f = null;
			for(int i=0; i<this.fileFilters.length;i++){
				f = new SelectFileFilter(chooser, this.fileFilters[i]);
				chooser.addChoosableFileFilter(f);
			}
			int returnVal = chooser.showOpenDialog(this);
			if(returnVal == JFileChooser.APPROVE_OPTION){
			 	this.fName = chooser.getSelectedFile().toString();
			 	FileFilter filter = chooser.getFileFilter();
			 	this.getTFile().setText(fName);
			 	lastPath = chooser.getCurrentDirectory().getAbsolutePath();
			 	
			 	if (PluginServices.getMainFrame() == null) {
                    ((JDialog) (getParent().getParent().getParent()
                            .getParent())).dispose();
                } else {
                	//Creamos la capa y la cargamos
                	IProjection proj = this.getPProyection().getCurProj();
                	try{
                		View  theView = null;
            			try{
            				theView = (View)PluginServices.getMDIManager().getActiveWindow();
            			}catch(ClassCastException exc){
            				return;
            			}
                		                		
                		com.hardcode.driverManager.Driver driver = LayerFactory.getDM().getDriver("gvSIG Image Driver");
                      		
                		File fich = new File(fName);
                		
						ViewPort viewPort = theView.getMapControl().getMapContext().getViewPort();
						
						Extent ext = calcTempExtent(fName, 
													viewPort, 
													proj);
						
                		this.setLyrName("*"+fName.substring(fName.lastIndexOf(File.separator)+1));
                		lyrGeoRaster = this.createLayer("*"+fName.substring(fName.lastIndexOf(File.separator)+1, fName.length()),
                            (RasterDriver) driver, fich, proj);
                		
                		if(!this.getCbUseGeoref().isSelected())
                			lyrGeoRaster.setAssignExtent(ext);                			
                		
                			                	
	                	if(lyrGeoRaster != null){	        
	                		theView.getMapControl().getMapContext().beginAtomicEvent();
	                		theView.getMapControl().getMapContext().getLayers().addLayer(lyrGeoRaster);
	                		theView.getMapControl().getMapContext().endAtomicEvent();
	                		lyrGeoRaster.getStackZoom().setInitExtent(ext, true);
	                		lyrGeoRaster.setActive(true);
	                	}
                		
                	}catch(DriverLoadException exc){
                		NotificationManager.addError("No se pudo acceder a los drivers", exc);
                	}catch(DriverException exc){
                		NotificationManager.addError("El driver ha producido un error", exc);
                	}
                  
                }
			 }
	    }	
		
	}
	
	/**
	 * @return Returns the lyrRaster.
	 */
	public FLyrGeoRaster getFLyrGeoRaster() {
		return lyrGeoRaster;
	}

	/**
	 * @param lyrRaster The lyrRaster to set.
	 */
	public void setFLyrGeoRaster(FLyrGeoRaster lyrGeoRaster) {
		this.lyrGeoRaster = lyrGeoRaster;
	}
   
	/**
	 * 
	 * @author Nacho Brodin (brodin_ign@gva.es)
	 *
	 */
	class SelectFileFilter extends javax.swing.filechooser.FileFilter {
		
		private JFileChooser chooser = null;
		private String file = null;
		
		public SelectFileFilter(JFileChooser ch, String file){
			this.chooser = ch;
			this.file = file;
		}
		
	    public boolean accept(File f) {

	    	return f.isDirectory() || f.getName().toLowerCase().endsWith("."+file);
	    }
	    
	    public String getDescription() {
	    	return file;
	    }
	    
	}
	
	/**
	 * @return Returns the fName.
	 */
	public String getLyrName() {
		return lyrName;
	}
	/**
	 * @param name The fName to set.
	 */
	public void setLyrName(String name) {
		lyrName = name;
	}
	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getPCheckUseGeoref() {
		if (pCheckUseGeoref == null) {
			FlowLayout flowLayout1 = new FlowLayout();
			flowLayout1.setAlignment(java.awt.FlowLayout.LEFT);
			lUseGeoref = new JLabel();
			pCheckUseGeoref = new JPanel();
			pCheckUseGeoref.setLayout(flowLayout1);
			pCheckUseGeoref.setPreferredSize(new java.awt.Dimension(370,31));
			lUseGeoref.setText("Usar georeferenciacin de la imagen");
			pCheckUseGeoref.add(getCbUseGeoref(), null);
			pCheckUseGeoref.add(lUseGeoref, null);
		}
		return pCheckUseGeoref;
	}
	/**
	 * This method initializes jCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */    
	private JCheckBox getCbUseGeoref() {
		if (cbUseGeoref == null) {
			cbUseGeoref = new JCheckBox();
			cbUseGeoref.setSelected(true);
		}
		return cbUseGeoref;
	}
   }
