package com.iver.cit.gvsig.jdbc_spatial.gui.jdbcwizard;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.FlowLayout;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.TreeMap;

import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.cresques.cts.IProjection;
import org.gvsig.gui.beans.swing.JButton;

import com.hardcode.driverManager.DriverLoadException;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.cit.gvsig.fmap.core.ICanReproject;
import com.iver.cit.gvsig.fmap.crs.CRSFactory;
import com.iver.cit.gvsig.fmap.drivers.DBLayerDefinition;
import com.iver.cit.gvsig.fmap.drivers.VectorialJDBCDriver;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.gui.WizardPanel;
import com.iver.utiles.NotExistInXMLEntity;
import com.iver.utiles.XMLEntity;
/**
 * @author Fernando Gonzlez Corts
 */
public class WizardJDBC extends WizardPanel{

	private JPanel pnlWizard = null;
	private JPanel jPanel1 = null;
	private JButton btnBack = null;
	private JButton btnNext = null;
	private ConnectionPanel connectionPanel = null;
	private DBLayerDefinitionPanel dbLayerDefinition = null;
	private FieldSelection fieldSelection = null;

	private int step = 0;
	private final int nsteps = 5;
	private static final String CONNECTION = "conn";
	private static final String LAYER_DEFINITION = "layerdef";
	private static final String FIELD_SELECTION= "fieldsel";
	private VectorialJDBCDriver driver;

	private GeomFieldSelection geomFieldSelection = null;
	private UniqueFieldSelection uniqueFieldSelection = null;

    private DatabaseMetaData dbmd = null;
    private String catalog = null;
    private String selectTable = null;
    private String[] theTables = null;
    private Connection conex = null;

	private HashMap settings = new HashMap();
	/**
	 * This is the default constructor
	 */
	public WizardJDBC() {
		super();
		initialize();
	}
	/**
	 * This method initializes this
	 *
	 * @return void
	 */
	private  void initialize() {
		setTabName("JDBC");
		this.setLayout(new BorderLayout());
		this.setSize(300, 270);
		this.add(getPnlWizard(), java.awt.BorderLayout.CENTER);
		this.add(getJPanel1(), java.awt.BorderLayout.SOUTH);
		connectionPanel.setDrivers(getDriverNames());
		enableButtons();


        XMLEntity xml = PluginServices.getPluginServices(this)
                                      .getPersistentXML();

        if (xml == null) {
            xml = new XMLEntity();
        }

        if (!xml.contains("jdbc-connections")) {
            String[] servers = new String[0];
            xml.putProperty("jdbc-connections", servers);
        }

        try {
            String[] servers = xml.getStringArrayProperty("jdbc-connections");

            for (int i = 0; i < servers.length; i++) {
                ConnectionSettings cs = new ConnectionSettings();
                cs.setFromString(servers[i]);
                settings.put(cs.getName(), cs);
            }
            getConnectionPanel().setSettings(settings);
        } catch (NotExistInXMLEntity e) {
        }

	}

	private String[] getDriverNames(){
		Class[] classes = new Class[] { VectorialJDBCDriver.class };

		ArrayList ret = new ArrayList();
		String[] driverNames = LayerFactory.getDM().getDriverNames();

		for (int i = 0; i < driverNames.length; i++) {
			boolean is = false;

			for (int j = 0; j < classes.length; j++) {
				if (LayerFactory.getDM().isA(driverNames[i], classes[j])) {
					ret.add(driverNames[i]);
				}
			}
		}

		return (String[]) ret.toArray(new String[0]);

	}

	private void enableButtons(){
		getBtnBack().setEnabled(step > 0);
		getBtnNext().setEnabled(step < nsteps - 1);
	}

	/**
	 * This method initializes pnlWizard
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getPnlWizard() {
		if (pnlWizard == null) {
			pnlWizard = new JPanel();
			pnlWizard.setLayout(new CardLayout());
			pnlWizard.add(getConnectionPanel(), CONNECTION);
			pnlWizard.add(getDbLayerDefinition(), LAYER_DEFINITION);
			pnlWizard.add(getFieldSelection(), FIELD_SELECTION);
			pnlWizard.add(getGeomFieldSelection(), getGeomFieldSelection().getName());
			pnlWizard.add(getUniqueFieldSelection(), getUniqueFieldSelection().getName());
		}
		return pnlWizard;
	}
	/**
	 * This method initializes jPanel1
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			FlowLayout flowLayout1 = new FlowLayout(FlowLayout.RIGHT);
			jPanel1 = new JPanel();
			jPanel1.setLayout(flowLayout1);
			jPanel1.setPreferredSize(new java.awt.Dimension(10,30));
			flowLayout1.setHgap(5);
			flowLayout1.setVgap(4);
			jPanel1.add(getBtnBack(), null);
			jPanel1.add(getBtnNext(), null);
		}
		return jPanel1;
	}
	/**
	 * This method initializes btnBack
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getBtnBack() {
		if (btnBack == null) {
			btnBack = new JButton();
			btnBack.setText(PluginServices.getText(this, "back"));
			//btnBack.setPreferredSize(new java.awt.Dimension(93,18));
			btnBack.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					step--;
					enableButtons();
					((CardLayout)pnlWizard.getLayout()).previous(pnlWizard);
				}
			});
		}
		return btnBack;
	}
	/**
	 * This method initializes btnNext
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getBtnNext() {
		if (btnNext == null) {
			btnNext = new JButton();
			btnNext.setText(PluginServices.getText(this, "next"));
			//btnNext.setPreferredSize(new java.awt.Dimension(93,18));
			btnNext.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					boolean done = false;
					if (step == 0) done = connectionPanel.done();
					else if (step == 1) done = dbLayerDefinition.done();
					else if (step == 2) done = fieldSelection.done();
					else if (step == 3) done = geomFieldSelection.done();
					if (done){
						try {
							if (step == 0){
                                catalog = connectionPanel.getDBName();
                                theTables = getTableNames();
								dbLayerDefinition.setTables(theTables);
							    saveConnection();
							}
							else if (step == 1) {
								fieldSelection.setFields(getTableFields());
							}else if (step == 2) {
								geomFieldSelection.setFields(getTableFields());
							}else if (step == 3) {
								uniqueFieldSelection.setFields(getTableFields());
							}
							step++;
							enableButtons();
							((CardLayout)pnlWizard.getLayout()).next(pnlWizard);
						} catch (SQLException e1) {
							NotificationManager.addError(PluginServices.getText(this,"error_conexion"), e1);
						} catch (DriverLoadException e1) {
							NotificationManager.addError("No se pudo cargar el driver", e1);
						}
					}else{
						JOptionPane.showMessageDialog(WizardJDBC.this, "No estan todos los datos rellenos", "Error", JOptionPane.ERROR_MESSAGE);

					}
				}
			});
		}
		return btnNext;
	}

    private void saveConnection() {
        connectionPanel.saveConnectionSettings();

        // settings.put(connectionPanel.getSettingsName(), cs);


    }

    private String[] getTableNames() throws SQLException, DriverLoadException {
		conex = DriverManager.getConnection(getConnectionString(), connectionPanel.getUser(),
				connectionPanel.getPassword());

		dbmd = conex.getMetaData();
        String[] types = {"TABLE", "VIEW"};
		ResultSet rs = dbmd.getTables(catalog, null, null, types);
		TreeMap ret = new TreeMap();
		while (rs.next()){
			ret.put(rs.getString("TABLE_NAME"), rs.getString("TABLE_NAME"));
		}

		return (String[]) ret.keySet().toArray(new String[0]);
	}

	private String[] getTableFields() throws SQLException, DriverLoadException{
		Statement st = conex.createStatement();
        // ResultSet rs = dbmd.getTables(catalog, null, dbLayerDefinition.getTable(), null);
		ResultSet rs = st.executeQuery("select * from " + dbLayerDefinition.getTable() + " LIMIT 1");
		ResultSetMetaData rsmd = rs.getMetaData();

		String[] ret = new String[rsmd.getColumnCount()];

		for (int i = 0; i < ret.length; i++) {
			ret[i] = rsmd.getColumnName(i+1);
		}

		return ret;
	}

	/**
	 * This method initializes connectionPanel
	 *
	 * @return com.iver.cit.gvsig.gui.Panels.ConnectionPanel
	 */
	private ConnectionPanel getConnectionPanel() {
		if (connectionPanel == null) {
			connectionPanel = new ConnectionPanel();
			connectionPanel.setName("connectionPanel");
			connectionPanel.setDrivers(getDriverNames());
		}
		return connectionPanel;
	}
	/**
	 * This method initializes dbLayerDefinition
	 *
	 * @return com.iver.cit.gvsig.gui.Panels.dbLayerDefinition
	 */
	private DBLayerDefinitionPanel getDbLayerDefinition() {
		if (dbLayerDefinition == null) {
			dbLayerDefinition = new DBLayerDefinitionPanel();
			dbLayerDefinition.setName("dbLayerDefinition");
		}
		return dbLayerDefinition;
	}
	/**
	 * This method initializes fieldSelection
	 *
	 * @return com.iver.cit.gvsig.gui.Panels.FieldSelection
	 */
	private FieldSelection getFieldSelection() {
		if (fieldSelection == null) {
			fieldSelection = new FieldSelection();
			fieldSelection.setName("fieldSelection");
		}
		return fieldSelection;
	}

	public VectorialJDBCDriver getDriver() throws DriverLoadException{
		if (driver == null){
			driver = (VectorialJDBCDriver) LayerFactory.getDM().getDriver(connectionPanel.getDriver());
		}

		return driver;
	}

	public String getConnectionString() throws DriverLoadException{
		String connectionString = getDriver().getConnectionStringBeginning() + "//" + connectionPanel.getHost();

		if (connectionPanel.getPort().trim().length() > 0) {
			connectionString += (":" + connectionPanel.getPort());
		} else {
		    connectionString += (":" + driver.getDefaultPort());
		}

		connectionString += ("/" + connectionPanel.getDBName());

		return connectionString;
	}

	public String getLayerName(){
		return dbLayerDefinition.getLayerName();
	}

	/**
	 * @return ONLY alphanumeric fields. You need to retrieve
     * the geometry fields with getGeometryField();
	 * @throws DriverLoadException
	 */
	public String[] getFields() throws DriverLoadException{
		String[] fields = fieldSelection.getFields();
		String geomField = geomFieldSelection.getField();
		String[] onlyAlphanumericFields = new String[fields.length-1];
        int newIndex = 0;
		for (int i = 0; i < fields.length; i++) {
			if (!fields[i].equals(geomField))
            {
				onlyAlphanumericFields[newIndex] = fields[i];
                newIndex++;
            }
		}
		return onlyAlphanumericFields;
	}

	/**
	 * @return WhereClause (doesn't check if correct)
	 */
	public String getWhereClause()
	{
		return dbLayerDefinition.getWhereClause();
	}
	/**
	 * @return
	 */
	public String getUser() {
		return connectionPanel.getUser();
	}
	/**
	 * @return
	 */
	public String getPassword() {
		return connectionPanel.getPassword();
	}
	/**
	 * @return
	 */
	public String getTable() {
		return dbLayerDefinition.getTable();
	}

	/**
	 * @return Field ID (Unique Value Field we want to use has key)
	 */
	public String getFID() {
		return uniqueFieldSelection.getField();
	}

    public String getGeomField()
    {
        return geomFieldSelection.getField();
    }

	/**
	 * This method initializes geomFieldSelection
	 *
	 * @return com.iver.cit.gvsig.gui.jdbcwizard.GeomFieldSelection
	 */
	private GeomFieldSelection getGeomFieldSelection() {
		if (geomFieldSelection == null) {
			geomFieldSelection = new GeomFieldSelection();
			geomFieldSelection.setName("geomFieldSelection");
		}
		return geomFieldSelection;
	}
	/**
	 * This method initializes uniqueFieldSelection
	 *
	 * @return com.iver.cit.gvsig.gui.jdbcwizard.UniqueFieldSelection
	 */
	private UniqueFieldSelection getUniqueFieldSelection() {
		if (uniqueFieldSelection == null) {
			uniqueFieldSelection = new UniqueFieldSelection();
			uniqueFieldSelection.setName("uniqueFieldSelection");
			uniqueFieldSelection.setWizard(this);
		}
		return uniqueFieldSelection;
	}
	public void initWizard() {
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.WizardPanel#execute()
	 */
	public void execute() {
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.WizardPanel#getLayer()
	 */
	public FLayer getLayer() {
		WizardJDBC wiz = this;
	    //true -> Modo desconectado. Por ahora, no se usa
        IProjection proj = null;
        String dbURL;
        try {
            dbURL = wiz.getConnectionString();
            String user = wiz.getUser();
            String pwd = wiz.getPassword();
            String layerName = wiz.getLayerName();
            String[] fields = wiz.getFields();
            String tableName = wiz.getTable();
            String whereClause = wiz.getWhereClause();
            String fidField = wiz.getFID();
            String geomField = wiz.getGeomField();
            Connection conn = DriverManager.getConnection(dbURL, user, pwd);
            conn.setAutoCommit(false);

            VectorialJDBCDriver driver = wiz.getDriver();
            if (dbLayerDefinition.getWorkingArea() != null){
                driver.setWorkingArea(dbLayerDefinition.getWorkingArea());
            }
            String strEPSG = getMapCtrl().getViewPort()
                        .getProjection().getAbrev()
                        .substring(5);
            DBLayerDefinition lyrDef = new DBLayerDefinition();
            lyrDef.setName(layerName);
            lyrDef.setTableName(tableName);
            lyrDef.setWhereClause(whereClause);
            lyrDef.setFieldNames(fields);
            lyrDef.setFieldGeometry(geomField);
            lyrDef.setFieldID(fidField);
            if (dbLayerDefinition.getWorkingArea() != null)
                lyrDef.setWorkingArea(dbLayerDefinition.getWorkingArea());

            lyrDef.setSRID_EPSG(strEPSG);
            if (driver instanceof ICanReproject)
            {
                ((ICanReproject)driver).setDestProjection(strEPSG);
            }
            driver.setData(conn, lyrDef);
            if (driver instanceof ICanReproject)
            {
                proj = CRSFactory.getCRS("EPSG:" + ((ICanReproject)driver).getSourceProjection());
            }

    	    if (false){
                FLayer lyr = LayerFactory.createDisconnectedDBLayer(driver, layerName, proj, null);
    			if (lyr != null) {
    				lyr.setVisible(true);
    			}

    			return lyr;
    	    }else{ // MODO CONECTADO
                return LayerFactory.createDBLayer(driver, layerName, proj);
    	    }
        } catch (Exception e) {
            e.printStackTrace();
            NotificationManager.addError("Error al cargar la capa.", e);
        }

		return null;
	}
}
