/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.core.mdiManager;

import java.awt.Component;
import java.awt.Image;
import java.util.Hashtable;
import java.util.Iterator;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JPanel;

import com.iver.andami.ui.mdiFrame.MDIFrame;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.WindowInfo;


/**
 *
 */
public class FrameWindowSupport {
    private Hashtable frameView = new Hashtable();
    private Hashtable viewFrame = new Hashtable();
    private Image icon;
    private WindowInfoSupport vis;
	private JFrame mainFrame;

    /**
     * Creates a new FrameViewSupport object.
     *
     * @param i DOCUMENT ME!
     */
    public FrameWindowSupport(MDIFrame mainFrame) {
    	this.mainFrame = mainFrame;
        icon = mainFrame.getIconImage();
    }

    public Iterator getWindowIterator(){
    	return viewFrame.keySet().iterator();
    }
    
    public boolean contains(IWindow v){
    	return viewFrame.containsKey(v);
    }

	/**
	 * @param wnd
	 * @return
	 */
	public boolean contains(JInternalFrame wnd) {
		return frameView.contains(wnd);
	}
    
    /**
     * DOCUMENT ME!
     *
     * @param p DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public JDialog getJDialog(IWindow p) {
        JDialog dlg = (JDialog) viewFrame.get(p);

        if (dlg == null) {
            WindowInfo vi = vis.getWindowInfo(p);
            JDialog nuevo = new JDialog(mainFrame);

            nuevo.getContentPane().add((JPanel) p);
            nuevo.setSize(getWidth(p, vi), getHeight(p, vi) + 30);
            nuevo.setTitle(vi.getTitle());
            nuevo.setResizable(vi.isResizable());

            viewFrame.put(p, nuevo);
            frameView.put(nuevo, p);

            nuevo.setModal(vi.isModal());
            return nuevo;
        } else {
            return dlg;
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param p DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public JInternalFrame getJInternalFrame(IWindow p) {
    	JInternalFrame frame = (JInternalFrame) viewFrame.get(p);

        if (frame == null) {
        	//ViewInfo vi = vis.getViewInfo(p);
            JInternalFrame nuevo = createJInternalFrame(p);
            viewFrame.put(p, nuevo);
            frameView.put(nuevo, p);

            return nuevo;
        } else {
            return frame;
        }
    }

    public JInternalFrame createJInternalFrame(IWindow p)
    {
        WindowInfo vi = vis.getWindowInfo(p);
        JInternalFrame nuevo = new JInternalFrame();
        if (icon != null){
            nuevo.setFrameIcon(new ImageIcon(icon));
        }
        nuevo.getContentPane().add((JPanel) p);
        nuevo.setClosable(true);
        nuevo.setSize(getWidth(p, vi), getHeight(p, vi));
        nuevo.setTitle(vi.getTitle());
        nuevo.setVisible(vi.isVisible());
        nuevo.setResizable(vi.isResizable());
        nuevo.setIconifiable(vi.isIconifiable());
        nuevo.setMaximizable(vi.isMaximizable());
        nuevo.setLocation(vi.getX(), vi.getY());

        nuevo.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
        return nuevo;
    }
    
    public IWindow getWindow(Component dlg){
    	return (IWindow) frameView.get(dlg);
    }
    
    public Component getFrame(IWindow window) {
    	return (Component) viewFrame.get(window);
    }
    
    public void closeWindow(IWindow v){
    	Object c = viewFrame.remove(v);
    	frameView.remove(c);
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     * @param x DOCUMENT ME!
     */
    public void setX(IWindow v, int x) {
    	Object frame = (JInternalFrame) viewFrame.get(v);
    	if (frame instanceof JInternalFrame) {
    		JInternalFrame iframe = (JInternalFrame) frame;
    		iframe.setLocation(x, iframe.getY());
    	}
    	else if (frame instanceof JDialog) {
    		JDialog dialog = (JDialog) frame;
    		dialog.setLocation(x, dialog.getY());
    	}
    }

    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     * @param y DOCUMENT ME!
     */
    public void setY(IWindow v, int y) {
    	Object frame = (JInternalFrame) viewFrame.get(v);
    	if (frame instanceof JInternalFrame) {
    		JInternalFrame iframe = (JInternalFrame) frame;
    		iframe.setLocation(iframe.getX(), y);
    	}
    	else if (frame instanceof JDialog) {
    		JDialog dialog = (JDialog) frame;
    		dialog.setLocation(dialog.getX(), y);
    	}
    }

    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     * @param height DOCUMENT ME!
     */
    public void setHeight(IWindow v, int height) {
    	Object frame = (JInternalFrame) viewFrame.get(v);
    	if (frame instanceof JInternalFrame) {
    		JInternalFrame iframe = (JInternalFrame) frame;
    		iframe.setSize(iframe.getWidth(), height);
    	}
    	else if (frame instanceof JDialog) {
    		JDialog dialog = (JDialog) frame;
    		dialog.setSize(dialog.getWidth(), height);
    	}
    }

    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     * @param width DOCUMENT ME!
     */
    public void setWidth(IWindow v, int width) {
    	Object frame = (JInternalFrame) viewFrame.get(v);
    	if (frame instanceof JInternalFrame) {
    		JInternalFrame iframe = (JInternalFrame) frame;
    		iframe.setSize(width, iframe.getHeight());
    	}
    	else if (frame instanceof JDialog) {
    		JDialog dialog = (JDialog) frame;
    		dialog.setSize(width, dialog.getHeight());
    	}
    }

    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     * @param title DOCUMENT ME!
     */
    public void setTitle(IWindow v, String title) {
    	Object frame = (JInternalFrame) viewFrame.get(v);
    	if (frame instanceof JInternalFrame)
    		((JInternalFrame)frame).setTitle(title);
    	else if (frame instanceof JDialog)
    		((JDialog)frame).setTitle(title);
    }

    /**
     * DOCUMENT ME!
     *
     * @param vis The vis to set.
     */
    public void setVis(WindowInfoSupport vis) {
        this.vis = vis;
    }

    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    private int getWidth(IWindow v) {
        WindowInfo vi = vis.getWindowInfo(v);

        if (vi.getWidth() == -1) {
            JPanel p = (JPanel) v;

            return p.getSize().width;
        } else {
            return vi.getWidth();
        }
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    private int getWidth(IWindow v, WindowInfo wi) {
        if (wi.getWidth() == -1) {
            JPanel p = (JPanel) v;

            return p.getSize().width;
        } else {
            return wi.getWidth();
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    private int getHeight(IWindow v) {
        WindowInfo vi = vis.getWindowInfo(v);

        if (vi.getHeight() == -1) {
            JPanel p = (JPanel) v;

            return p.getSize().height;
        } else {
            return vi.getHeight();
        }
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param v DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    private int getHeight(IWindow v, WindowInfo wi) {
        if (wi.getHeight() == -1) {
            JPanel p = (JPanel) v;

            return p.getSize().height;
        } else {
            return wi.getHeight();
        }
    }
    
    public void updateWindowInfo(IWindow win, WindowInfo windowInfo) {
    	Object o = viewFrame.get(win);
    	if (windowInfo!=null && o!=null) {
    		if (o instanceof JComponent) {
        		JComponent component = (JComponent) o;
        		windowInfo.setWidth(component.getWidth());
				windowInfo.setHeight(component.getHeight());
				windowInfo.setX(component.getX());
				windowInfo.setY(component.getY());
				windowInfo.setClosed(!component.isShowing());
				if (component instanceof JInternalFrame) {
					JInternalFrame iframe = (JInternalFrame) component;
					windowInfo.setNormalBounds(iframe.getNormalBounds());
					windowInfo.setMaximized(iframe.isMaximum());
				}
    		}
    	}
    }
}
