/*
 * Created on 23-mar-2006
 * 
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 */
package org.cresques.filter;

import org.cresques.io.GeoRasterFile;


/**
 * Este interface lo implementaran las clases que se registran en propiedades
 * de raster. Representa a las operaciones que pueden realizar y que deben ser
 * invocadas desde la libreria de cresques. Cuando las propiedades del raster
 * ejecutan alguna de estas operaciones recorreran todas las clases que han sido
 * registradas y ejecutaran para cada una la operacin. Si la clase registrada no
 * necesita hacer nada con esa operacin simplemente no la implementar.
 *  
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public interface IRasterOperationsRegistrablePanels{
	
	/**
	 * Operacin de aadir ficheros. El dialogo para aadir bandas se encuentra
	 * en cresques. Es corriente que haya paneles que necesiten ser informados
	 * cuando se aaden nuevas bandas. Si es as addFiles debera ser implementado.
	 * @param files
	 */
	public void addFiles(GeoRasterFile[] files);
	
	/**
	 * Operacin de eliminar ficheros. El dialogo para eliminar bandas se encuentra
	 * en cresques. Es corriente que haya paneles que necesiten ser informados
	 * cuando se eliminan bandas. Si es as removeFile debera ser implementado.
	 * @param files
	 */
	public void removeFile(String file);
}