/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.drivers;

import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.TreeMap;

import org.apache.log4j.Logger;

import com.hardcode.driverManager.DriverEvent;
import com.hardcode.driverManager.DriverEventListener;
import com.hardcode.driverManager.IDelayedDriver;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.driver.ObjectDriver;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.DriverJdbcNotFoundExceptionType;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.Messages;
import com.iver.cit.gvsig.fmap.SqlDriveExceptionType;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.drivers.jdbc.utils.ConnectionWithParams;
import com.iver.cit.gvsig.fmap.drivers.jdbc.utils.SingleJDBCConnectionManager;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.XMLException;
import com.iver.utiles.XMLEntity;
import com.iver.utiles.swing.JPasswordDlg;



/**
 * Clase abstracta qu
 */
public abstract class DefaultDBDriver implements VectorialJDBCDriver, ObjectDriver {
    private static Logger logger = Logger.getLogger(SelectableDataSource.class.getName());
    protected static Hashtable poolPassw = new Hashtable();

    protected Connection conn;
    // protected String tableName;
    // protected String whereClause;
    // protected String fields;
    // protected String sqlOrig;
    protected DBLayerDefinition lyrDef = null;
    protected ResultSet rs;
    protected boolean bCursorActivo = false;
    protected Statement st;
    protected int numReg=-1;

    protected Rectangle2D fullExtent = null;

    // protected String strFID_FieldName;
    // protected String idFID_FieldName;

    protected Hashtable hashRelate;


    protected ResultSetMetaData metaData = null;
    protected Rectangle2D workingArea;
    protected String driverClass;
    protected String userName;
    protected String dbUrl;
    protected String className;
    protected String catalogName;
    protected String tableName;
    protected String[] fields;
    protected String FIDfield;
    protected String geometryField;
    protected String whereClause;
    protected String strSRID;
	//private double flatness;
	
    protected String host, port, dbName, connName;
    
    protected ArrayList driverEventListeners = new ArrayList();
    

    abstract public void setData(Connection conn, DBLayerDefinition lyrDef);

	/**
	 * @return devuelve la Conexin a la base de datos, para que
	 * el usuario pueda hacer la consulta que quiera, si lo desea.
	 * Por ejemplo, esto puede ser til para abrir un cuadro de dialogo
	 * avanazado y lanzar peticiones del tipo "Devuelveme un buffer
	 * a las autopistas", y con el resultset que te venga, escribir
	 * un shape, o cosas as.
	 */
	public Connection getConnection()
	{
	    return conn;
	}
	public String[] getFields()
	{
        /* StringTokenizer tokenizer = new StringTokenizer(fields, ",");
        String[] arrayFields = new String[tokenizer.countTokens()];
        int i=0;
        while (tokenizer.hasMoreTokens())
        {
            arrayFields[i] = tokenizer.nextToken();
            i++;
        }
	    return arrayFields; */
        return lyrDef.getFieldNames();

	}
    /**
     * First, the geometry field. After, the rest of fields
     * @return
     */
    public String getTotalFields()
    {
        String strAux = getGeometryField(getLyrDef().getFieldGeometry());
        String[] fieldNames = getLyrDef().getFieldNames();
        for (int i=0; i< fieldNames.length; i++)
        {
            strAux = strAux + ", " + fieldNames[i];
        }
        return strAux;
    }

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDatabaseDriver#getWhereClause()
	 */
	public String getWhereClause()
	{
	    return lyrDef.getWhereClause().toUpperCase();
	}
	public String getTableName()
	{
	    return lyrDef.getTableName();
	}


	/**
	 * @throws DriverIOException
	 * @throws DriverException
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#getShapeCount()
	 */
	public int getShapeCount() throws IOException {
	    	if (numReg == -1)
	    	{
	    	    try
	            {
	    	        Statement s = conn.createStatement();
		            ResultSet r = s.executeQuery("SELECT COUNT(*) AS NUMREG FROM " + lyrDef.getTableName() + " " + getCompleteWhere());
		            r.next();
		            numReg = r.getInt(1);
		            System.err.println("numReg = " + numReg);
	            }
	    	    catch (SQLException e)
	    	    {
	    	        throw new IOException(e.getMessage());
	    	    }
	    	}

            return numReg;
	}

    /**
     * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#getFullExtent()
     */
    public Rectangle2D getFullExtent(){
        // Por defecto recorremos todas las geometrias.
        // Las bases de datos como PostGIS pueden y deben
        // sobreescribir este mtodo.
        if (fullExtent == null)
        {
            try
            {
                IFeatureIterator itGeom = getFeatureIterator("SELECT " +
                        getGeometryField(getLyrDef().getFieldGeometry()) + ", " + getLyrDef().getFieldID() + " FROM " +
                        getLyrDef().getTableName() +  " " + getCompleteWhere());
                IGeometry geom;
                int cont = 0;
                while (itGeom.hasNext())
                {
                    geom = itGeom.next().getGeometry();
                    if (cont==0)
                        fullExtent = geom.getBounds2D();
                    else
                        fullExtent.add(geom.getBounds2D());
                    cont++;
                }
            }
            catch (DriverException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }

        }
        return fullExtent;
    }


	/**
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#getShapeType()
	 */
	public int getShapeType() {
        /* IGeometry geom;
        if (shapeType == -1)
        {
        	shapeType = FShape.MULTI;
        	try {
        		geom = getShape(0);
        		if (geom != null)
        			shapeType = geom.getGeometryType();
        	} catch (IOException e) {
        		// e.printStackTrace();
        	}
        }
        return shapeType; */
		return lyrDef.getShapeType();
	}

	public int getFieldType(int idField) throws com.hardcode.gdbms.engine.data.driver.DriverException
	{
	    String str = "";
	    try {
	        int i = idField + 2; // idField viene basado en 1, y
	        			// adems nos saltamos el campo de geometry
	        str = metaData.getColumnClassName(i);
            if (metaData.getColumnType(i) == Types.VARCHAR)
                return Types.VARCHAR;
		    if (metaData.getColumnType(i) == Types.FLOAT)
		        return Types.FLOAT;
		    if (metaData.getColumnType(i) == Types.DOUBLE)
		        return Types.DOUBLE;
		    if (metaData.getColumnType(i) == Types.INTEGER)
		        return Types.INTEGER;
		    if (metaData.getColumnType(i) == Types.SMALLINT)
		        return Types.SMALLINT;
		    if (metaData.getColumnType(i) == Types.TINYINT)
		        return Types.TINYINT;		    
		    if (metaData.getColumnType(i) == Types.BIGINT)
		        return Types.BIGINT;
		    if (metaData.getColumnType(i) == Types.BIT)
		        return Types.BIT;
		    if (metaData.getColumnType(i) == Types.DATE)
		        return Types.DATE;
            if (metaData.getColumnType(i) == Types.DECIMAL)
                return Types.DOUBLE;
            if (metaData.getColumnType(i) == Types.NUMERIC)
                return Types.DOUBLE;
            if (metaData.getColumnType(i) == Types.DATE)
                return Types.DATE;
            if (metaData.getColumnType(i) == Types.TIME)
                return Types.TIME;
            if (metaData.getColumnType(i) == Types.TIMESTAMP)
                return Types.TIMESTAMP;

	    } catch (SQLException e) {
	    	throw new com.hardcode.gdbms.engine.data.driver.DriverException(e);
    	}
	    return Types.OTHER;
        // throw new com.hardcode.gdbms.engine.data.driver.DriverException("Tipo no soportado: " + str);
	}
    /**
     * Obtiene el valor que se encuentra en la fila y columna indicada
     * Esta es la implementacin por defecto. Si lo del absolute
     * no va bien, como es el caso del PostGis, el driver lo
     * tiene que reimplementar
     *
     * @param rowIndex fila
     * @param fieldId columna
     *
     * @return subclase de Value con el valor del origen de datos
     *
     * @throws DriverException Si se produce un error accediendo al DataSource
     */
    public Value getFieldValue(long rowIndex, int idField)
        throws com.hardcode.gdbms.engine.data.driver.DriverException
        {
        	int i = (int) (rowIndex + 1);
        	int fieldId = idField+2;
    	    try {
            	rs.absolute(i);
                if (metaData.getColumnType(fieldId) == Types.VARCHAR)
                {
                    String strAux = rs.getString(fieldId);
                    if (strAux == null) strAux = "";
                    return ValueFactory.createValue(strAux);
                }
    		    if (metaData.getColumnType(fieldId) == Types.FLOAT)
    		        return ValueFactory.createValue(rs.getFloat(fieldId));
    		    if (metaData.getColumnType(fieldId) == Types.DOUBLE)
    		        return ValueFactory.createValue(rs.getDouble(fieldId));
    		    if (metaData.getColumnType(fieldId) == Types.INTEGER)
    		        return ValueFactory.createValue(rs.getInt(fieldId));
    		    if (metaData.getColumnType(fieldId) == Types.BIGINT)
    		        return ValueFactory.createValue(rs.getLong(fieldId));
    		    if (metaData.getColumnType(fieldId) == Types.BIT)
    		        return ValueFactory.createValue(rs.getBoolean(fieldId));
    		    if (metaData.getColumnType(fieldId) == Types.DATE)
    		        return ValueFactory.createValue(rs.getDate(fieldId));
    	    } catch (SQLException e) {
                throw new com.hardcode.gdbms.engine.data.driver.DriverException("Tipo no soportado: columna " + fieldId );
        	}
        	return null;


        }

    /**
     * Obtiene el nmero de campos del DataSource
     *
     * @return
     *
     * @throws DriverException Si se produce algn error accediendo al
     *         DataSource
     */
    public int getFieldCount() throws com.hardcode.gdbms.engine.data.driver.DriverException
    {
        try {
            // Suponemos que el primer campo es el de las geometries, y no lo
            // contamos
            return rs.getMetaData().getColumnCount()-1;
        } catch (SQLException e) {
            throw new com.hardcode.gdbms.engine.data.driver.DriverException(e);
        }

    }

    /**
     * Devuelve el nombre del campo fieldId-simo
     *
     * @param fieldId ndice del campo cuyo nombre se quiere obtener
     *
     * @return
     * @throws com.hardcode.gdbms.engine.data.driver.DriverException
     *
     * @throws DriverException Si se produce algn error accediendo al
     *         DataSource
     */
    public String getFieldName(int fieldId) throws com.hardcode.gdbms.engine.data.driver.DriverException
    {
        try {
            return rs.getMetaData().getColumnName(fieldId+2);
        } catch (SQLException e) {
            throw new com.hardcode.gdbms.engine.data.driver.DriverException(e);
        }
    }

    /**
     * Obtiene el nmero de registros del DataSource
     *
     * @return
     *
     * @throws DriverException Si se produce algn error accediendo al
     *         DataSource
     */
    public long getRowCount()
    {
        try {
            return getShapeCount();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return -1;
    }

    public void close() {
    }

    /**
     * Recorre el recordset creando una tabla Hash que usaremos para
     * relacionar el nmero de un registro con su identificador nico.
     * Debe ser llamado en el setData justo despus de crear el recorset
     * principal
     * @throws SQLException
     */
    protected void doRelateID_FID() throws SQLException
    {
        hashRelate = new Hashtable();


        String strSQL = "SELECT " + getLyrDef().getFieldID() + " FROM " + getLyrDef().getTableName()
        + " " + getCompleteWhere() + " ORDER BY " + getLyrDef().getFieldID();
        Statement s = getConnection().createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
        ResultSet r = s.executeQuery(strSQL);
        int id=0;
        int gid;
        int index = 0;
        while (r.next())
        {
            String aux = r.getString(1);
            Value val = ValueFactory.createValue(aux);
            hashRelate.put(val, new Integer(index));
            System.out.println("ASOCIANDO CLAVE " + aux + " CON VALOR " + index);
            index++;
        }
        numReg = index;
        r.close();
        // rs.beforeFirst();

    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.drivers.VectorialDatabaseDriver#getRowIndexByFID(java.lang.Object)
     */
    public int getRowIndexByFID(IFeature FID)
    {
        int resul;
        // Object obj = FID.getAttribute(lyrDef.getIdFieldID());
        String theId = FID.getID();
        Value aux = ValueFactory.createValue(theId);
        // System.err.println("Mirando si existe " + aux.toString());
        if (hashRelate.containsKey(aux))
        {
        	Integer rowIndex = (Integer) hashRelate.get(aux);
        	resul = rowIndex.intValue();
        	// System.err.println("Row asociada a " + aux.toString() + ":" + resul);
        	return resul;
        }
        else
        	return -1;
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.drivers.VectorialDatabaseDriver#setXMLEntity(com.iver.utiles.XMLEntity)
     */
    public void setXMLEntity(XMLEntity xml) throws XMLException
    {

        className = xml.getStringProperty("className");
        
        catalogName = xml.getStringProperty("catalog");
        userName =xml.getStringProperty("username");
        driverClass =xml.getStringProperty("driverclass");
        tableName = xml.getStringProperty("tablename");
        fields = xml.getStringArrayProperty("fields");
        FIDfield = xml.getStringProperty("FID");
        geometryField = xml.getStringProperty("THE_GEOM");
        whereClause = xml.getStringProperty("whereclause");
        strSRID = xml.getStringProperty("SRID");

        if (xml.contains("host"))
        {
	        host = xml.getStringProperty("host");
	        port = xml.getStringProperty("port");
	        dbName = xml.getStringProperty("dbName");
	        connName = xml.getStringProperty("connName");
        }
        else
        {
        	// Por compatibilidad con versiones anteriores
        	dbUrl = xml.getStringProperty("dbURL");
        	extractParamsFromDbUrl(dbUrl);
        	
        }
        if (xml.contains("minXworkArea"))
        {
            double x = xml.getDoubleProperty("minXworkArea");
            double y = xml.getDoubleProperty("minYworkArea");
            double H = xml.getDoubleProperty("HworkArea");
            double W = xml.getDoubleProperty("WworkArea");
            workingArea = new Rectangle2D.Double(x,y,W,H);
        }

        DBLayerDefinition lyrDef = new DBLayerDefinition();
        lyrDef.setCatalogName(catalogName);
        lyrDef.setTableName(tableName);
        lyrDef.setFieldNames(fields);
        lyrDef.setFieldID(FIDfield);
        lyrDef.setFieldGeometry(geometryField);
        lyrDef.setWhereClause(whereClause);
        // lyrDef.setClassToInstantiate(driverClass);
        if (workingArea != null)
            lyrDef.setWorkingArea(workingArea);

        lyrDef.setSRID_EPSG(strSRID);

        setLyrDef(lyrDef);

    }

    private void extractParamsFromDbUrl(String dbUrl2) {
    	//jdbc:postgres://localhost:5431/latin1
    	int iDbName = dbUrl2.lastIndexOf('/');
		dbName = dbUrl2.substring(iDbName+1);
		int iLast2points = dbUrl2.lastIndexOf(':');
		port = dbUrl2.substring(iLast2points+1, iDbName);
		int iHost = dbUrl2.indexOf("//");
		host = dbUrl2.substring(iHost + 2, iLast2points);
		connName = dbUrl2;
	}

	public void load() throws DriverException {
    	try {
            if (driverClass != null)
            	Class.forName(driverClass);
            
            String _drvName = getName();

            String keyPool = _drvName.toLowerCase() + "_" + host.toLowerCase()
            + "_" + port + "_" + dbName.toLowerCase()
            + "_" + userName.toLowerCase();
            
            Connection newConn = null;
            String clave = null;
            ConnectionWithParams cwp = null;
            
            if (poolPassw.containsKey(keyPool)) {
            	
                clave = (String) poolPassw.get(keyPool);
                cwp = SingleJDBCConnectionManager.instance().getConnection(
 					_drvName, userName, clave, connName,
 					host, port, dbName, true);

            } else {
            	
                cwp = SingleJDBCConnectionManager.instance().getConnection(
     					_drvName, userName, null, connName,
     					host, port, dbName, false);
                
                if (cwp.isConnected()) {
                	
                	poolPassw.put(keyPool, cwp.getPw());
                	
                } else {
                	
                    JPasswordDlg dlg = new JPasswordDlg();
                    String strMessage = Messages.getString("conectar_jdbc");
                    String strPassword = Messages.getString("password");
                    dlg.setMessage(strMessage
                    		+ " ["
                    		+ _drvName + ", "
                    		+ host + ", "
                    		+ port + ", "
                    		+ dbName + ", "
                    		+ userName + "]. "
                    		+ strPassword
                    		+ "?");
                    dlg.show();
                    clave = dlg.getPassword();
                    if (clave == null)
                        return;
                    poolPassw.put(keyPool, clave);
                    
                    cwp.connect(clave);
                }
            }

            newConn = cwp.getConnection();
            newConn.setAutoCommit(false);

            DBLayerDefinition lyrDef = new DBLayerDefinition();
            if (getLyrDef() == null) {
	            lyrDef.setCatalogName(catalogName);
	            lyrDef.setTableName(tableName);
	            lyrDef.setFieldNames(fields);
	            lyrDef.setFieldID(FIDfield);
	            lyrDef.setFieldGeometry(geometryField);
	            lyrDef.setWhereClause(whereClause);
	            // lyrDef.setClassToInstantiate(driverClass);
	            if (workingArea != null)
	                lyrDef.setWorkingArea(workingArea);

	            lyrDef.setSRID_EPSG(strSRID);
            } else {
            	lyrDef = getLyrDef();
            }

            setData(newConn, lyrDef);
        } catch (ClassNotFoundException e) {
            logger.debug(e);
            DriverJdbcNotFoundExceptionType type =
            	new DriverJdbcNotFoundExceptionType();
            type.setDriverJdbcClassName(driverClass);
            type.setLayerName(this.getTableName());
            throw new DriverException("Driver JDBC no encontrado", e,  type);
        } catch (SQLException e) {
        	
        	throw new DriverException(e.getMessage());
        	
		}
    }
    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.drivers.VectorialDatabaseDriver#getXMLEntity()
     */
    public XMLEntity getXMLEntity()
    {
        XMLEntity xml = new XMLEntity();
        xml.putProperty("className",this.getClass().getName());
        try {
            DatabaseMetaData metadata = getConnection().getMetaData();
            xml.putProperty("catalog", getLyrDef().getCatalogName());
            
            // TODO: NO DEBEMOS GUARDAR EL NOMBRE DE USUARIO Y CONTRASEA
            // AQUI. Hay que utilizar un pool de conexiones
            // y pedir al usuario que conecte a la base de datos
            // en la primera capa. En el resto, usar la conexin
            // creada con anterioridad.
            String userName = metadata.getUserName();
            int aux = userName.indexOf("@");
            if (aux != -1)
                userName = userName.substring(0,aux);
            xml.putProperty("username", userName);

            Driver drv = DriverManager.getDriver(metadata.getURL());
            // System.out.println(drv.getClass().getName());
            xml.putProperty("driverclass", drv.getClass().getName());

            xml.putProperty("tablename", getTableName());
            xml.putProperty("fields", lyrDef.getFieldNames());
            xml.putProperty("FID", lyrDef.getFieldID());
            xml.putProperty("THE_GEOM", lyrDef.getFieldGeometry());
            xml.putProperty("whereclause", getWhereClause());
            xml.putProperty("SRID", lyrDef.getSRID_EPSG());

            ConnectionWithParams cwp =
            	SingleJDBCConnectionManager.instance().findConnection(getConnection());

            //FIXME:(Chema) Estos cambios los hago porque da errores as persistencia 
            if (cwp != null){
            	xml.putProperty("host", cwp.getHost());
            	xml.putProperty("port", cwp.getPort());
            	xml.putProperty("dbName", cwp.getDb());
            	xml.putProperty("connName", cwp.getName());
            } else {
            	xml.putProperty("dbURL",metadata.getURL());
            }
            // Chema
            
            if (getWorkingArea() != null)
            {
                xml.putProperty("minXworkArea", getWorkingArea().getMinX());
                xml.putProperty("minYworkArea", getWorkingArea().getMinY());
                xml.putProperty("HworkArea", getWorkingArea().getHeight());
                xml.putProperty("WworkArea", getWorkingArea().getWidth());
            }

        } catch (SQLException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }


        return xml;

    }

    /**
     * @see com.iver.cit.gvsig.fmap.drivers.VectorialJDBCDriver#setWorkingArea(java.awt.geom.Rectangle2D)
     */
    public void setWorkingArea(Rectangle2D rect) {
        this.workingArea = rect;
    }

    /**
     * @see com.iver.cit.gvsig.fmap.drivers.VectorialJDBCDriver#getWorkingArea()
     */
    public Rectangle2D getWorkingArea() {
        return workingArea;
    }

    /* (non-Javadoc)
     * @see com.hardcode.gdbms.engine.data.driver.GDBMSDriver#setDataSourceFactory(com.hardcode.gdbms.engine.data.DataSourceFactory)
     */
    public void setDataSourceFactory(DataSourceFactory arg0) {
        // TODO Auto-generated method stub

    }

    /**
     * @return Returns the lyrDef.
     */
    public DBLayerDefinition getLyrDef() {
    	if (this.conn != null){
    		if (lyrDef.getConnection() != this.conn){
    			lyrDef.setConnection(this.conn);
    		}
    	}
        return lyrDef;
    }

    /**
     * @param lyrDef The lyrDef to set.
     */
    public void setLyrDef(DBLayerDefinition lyrDef) {
        this.lyrDef = lyrDef;
    }

    abstract public String getSqlTotal();

    /**
     * @return Returns the completeWhere. WITHOUT order by clause!!
     */
    abstract public String getCompleteWhere();

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.drivers.VectorialDriver#reLoad()
     */
    public void reload() throws IOException
    {
		try {
	    	if ((conn == null) || (conn.isClosed()))
	    	{
	    		this.load();
	    	}

			conn.commit();

	    	setData(conn, lyrDef);
		} catch (SQLException e) {
			throw new IOException(e.getMessage());
		} catch (DriverException e) {
			throw new IOException(e.getMessage());
		}

    }

    public int getFieldWidth(int fieldId)
    {
    	int i = -1;
    	try {
    		int aux = fieldId + 2; // idField viene basado en 1, y
			i = rs.getMetaData().getColumnDisplaySize(aux);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		// SUN define que getColumnDisplaySize devuelve numeros negativos cuando el campo es de tipo Text o Vartext
		// sin ancho. Nosotros vamos a devolver 255 para que fucione, por lo menos al exportar a DBF.
		// Nota: Si se truncan cadenas, este es el sitio que lo provoca.
		if (i <0) i=255;
		return i;
    }

//	public void setFlatness(double flatness) {
//		this.flatness = flatness;
//	}
    
    // -----------------------------------------------------------
    // ----  EXT JDBC NUEVA                           ---
    // -----------------------------------------------------------
    
    /**
     * Gets the drivers connection string given its parameters (this can be different for
     * different driver, so it should be overwritten in that case.)
     */
    public String getConnectionString(
    		String host,
    		String port,
    		String dbname,
    		String user,
    		String pw) {
    	
		String resp = getConnectionStringBeginning() + "//" + host.toLowerCase();

		if (dbname.trim().length() > 0) {
			resp += ":" + port;
		} else {
			resp += ":" + getDefaultPort();
		}

		resp += "/" + dbname.toLowerCase();
		return resp;
    }
    
    /**
     * Gets available table names. Should be overwritten by subclasses if its
     * not compatible or if it can be refined
     * 
     * @param conn connection object
     * @param catalog catalog name
     * @return array of strings with available table names
     * @throws SQLException
     */
    public String[] getTableNames(Connection conn, String catalog) throws SQLException {
    	
    	DatabaseMetaData dbmd = conn.getMetaData();
        String[] types = {"TABLE", "VIEW"};
		ResultSet rs = dbmd.getTables(catalog, null, null, types);
		TreeMap ret = new TreeMap();
		while (rs.next()){
			ret.put(rs.getString("TABLE_NAME"), rs.getString("TABLE_NAME"));
		}
		rs.close();
		return (String[]) ret.keySet().toArray(new String[0]);
    }
    

    /**
     *       Gets all field names of a given table
     * @param conn connection object
     * @param table_name table name
     * @return all field names of the given table
     * @throws SQLException
     */
    public String[] getAllFields(Connection conn, String table_name) throws SQLException {
    	
		Statement st = conn.createStatement();
		ResultSet rs = st.executeQuery("select * from " + table_name + " LIMIT 1");
		ResultSetMetaData rsmd = rs.getMetaData();
		String[] ret = new String[rsmd.getColumnCount()];

		for (int i = 0; i < ret.length; i++) {
			ret[i] = rsmd.getColumnName(i+1);
		}
		rs.close(); st.close();
		return ret;
    }

    /**
     *       Gets all field type names of a given table
     * @param conn connection object
     * @param table_name table name
     * @return all field type names of the given table
     * @throws SQLException
     */
    public String[] getAllFieldTypeNames(Connection conn, String table_name) throws SQLException {
    	
		Statement st = conn.createStatement();
		ResultSet rs = st.executeQuery("select * from " + table_name + " LIMIT 1");
		ResultSetMetaData rsmd = rs.getMetaData();
		String[] ret = new String[rsmd.getColumnCount()];

		for (int i = 0; i < ret.length; i++) {
			ret[i] = rsmd.getColumnTypeName(i+1);
		}
		rs.close(); st.close();
		return ret;
    }

    /**
     * Gets the table's possible id fields. By default, all fields can be id. 
     * It should be overwritten by subclasses.
     * 
     * @param conn conenction object
     * @param table_name table name
     * @return the table's possible id fields
     * @throws SQLException
     */
    public String[] getIdFieldsCandidates(Connection conn, String table_name) throws SQLException {
    	return getAllFields(conn, table_name);
    }
    
    /**
     * Gets the table's possible geometry fields. By default, all fields can be geometry
     * fields. It should be overwritten by subclasses.
     * 
     * @param conn conenction object
     * @param table_name table name
     * @return the table's possible geometry fields
     * @throws SQLException
     */
    public String[] getGeometryFieldsCandidates(Connection conn, String table_name) throws SQLException {
    	return getAllFields(conn, table_name);
    }
    
    /**
     * Tells if it's an empty table (with no records)
     * @param conn conenction object
     * @param tableName rtable name
     * @return whether it's an empty table (with no records) or not
     */
	public boolean isEmptyTable(Connection conn, String tableName) {
		
		boolean res = true;

		try {
			Statement st = conn.createStatement();
			ResultSet rs = null;
			rs = st.executeQuery("select * from " + tableName + " LIMIT 1");
			res = !rs.next();
			rs.close(); st.close();
		} catch (Exception ex) {
			res = true;
		}
		return res;
	}



	/**
	 * Utility method to allow the driver to do something with the geometry field.
	 * By default does nothing.
	 * 
	 * @param flds user-selected fields
	 * @param geomField geometry field
	 * @return new user-selected fields
	 */
	public String[] manageGeometryField(String[] flds, String geomField) {
		return flds;
	}
	
	/**
	 * Empty method called when the layer is going to be removed from the view.
	 * Subclasses can overwrite it if needed.
	 *
	 */
	public void remove() {
		
	}

	public void addDriverEventListener(DriverEventListener listener) {
		if (!driverEventListeners.contains(listener))
			driverEventListeners.add(listener);
		
	}

	public void removeDriverEventListener(DriverEventListener listener) {
		driverEventListeners.remove(listener);
		
	}
	
	public void notifyDriverEndLoaded() {
		DriverEvent event = new DriverEvent(DriverEvent.DRIVER_EVENT_LOADING_END);
		for (int i=0; i < driverEventListeners.size(); i++)
		{
			DriverEventListener aux = (DriverEventListener) driverEventListeners.get(i);
			aux.driverNotification(event);
		}
	}
    
}
