package com.iver.cit.gvsig.fmap.edition;

import com.iver.cit.gvsig.fmap.core.IRow;

import java.io.IOException;

import java.util.BitSet;
import java.util.HashMap;


/**
 * Maneja el fichero de extensin en el que se almacenan las modificacionesy adicines
 * durante la edicin. Los ndices que devuelve esta clase en sus mtodos
 * addRow y modifyRow son invariables, es decir, que si se invoca un
 * mtodo addRow que retorna un 8, independientemente de las operaciones
 * que se realicen posteriormente, una llamada a getRow(8) retornar
 * dicha fila. Si esta geometra es eliminada posteriormente, se retornar un
 * null. Esto ltimo se cumple mientras no se invoque el mtodo compact, mediante
 * el que se reorganizan las geometras dejando en el fichero slo las que tienen
 * validez en el momento de realizar la invocacin.
 */
public interface ExpansionFile {
	/**
	 * Aade una geometria al final del fichero y retorna el ndice que ocupa
	 * esta geometria en el mismo
	 *
	 * @param row DOCUMENT ME!
	 * @param status TODO
	 * @param indexInternalFields fields that where valid when this row was added.
	 *
	 * @return calculated index of the new row.
	 *
	 * @throws IOException
	 */
	int addRow(IRow row, int status, int indexInternalFields) throws IOException;

	/**
	 * Modifica la index-sima geometra del fichero devolviendo la posicin en
	 * la que se pone la geometria modificada.
	 *
	 * @param calculated index of row to be modified
	 * @param row newRow that replaces the old row.
	 *
	 * @return new calculated index of the modified row.
	 *
	 * @throws IOException
	 */
	int modifyRow(int index, IRow row, int indexInternalFields) throws IOException;

	/**
	 * Obtiene la geometria que hay en el ndice 'index' o null si la geometra
	 * ha sido invalidada.
	 *
	 * @param index caculatedIndex of the row to be read.
	 * @return
	 * @throws IOException
	 */
	IRowEdited getRow(int index) throws IOException;

	/**
	 * Invalida una geometra, de cara a una futura compactacin del fichero
	 *
	 * @param index DOCUMENT ME!
	 */
	//void invalidateRow(int index);

	/**
	 * Realiza una compactacin del fichero que maneja esta clase
	 *
	 * @param relations DOCUMENT ME!
	 */
	void compact(HashMap relations);

	/**
	 * Devuelve el nmero de geometras del fichero.
	 *
	 * @return nmero de geometras.
	 */
	//int getRowCount();

    /**
     * Mueve el puntero de escritura de manera que las siguientes escrituras
     * machacarn la ltima fila
     */
    void deleteLastRow();

    /**
     * Abre el fichero de expansin para comenzar la edicin
     *
     * @throws IOException Si se produce algn error al abrir
     */
    void open() throws IOException;

    /**
     * Cierra el fichero de expansin al terminar la edicin
     *
     * @throws IOException Si se produce algn error
     */
    void close() throws IOException;

	/**
	 * @param previousExpansionFileIndex
	 */
	//void validateRow(int previousExpansionFileIndex);
	//BitSet getInvalidRows();
    
    int getSize();
}
