package com.iver.cit.gvsig.fmap.layers;

import org.cresques.cts.IProjection;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.instruction.FieldNotFoundException;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.crs.CRSFactory;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.VectorialDriver;
import com.iver.cit.gvsig.fmap.drivers.WithDefaultLegend;
import com.iver.cit.gvsig.fmap.rendering.LegendFactory;
import com.iver.cit.gvsig.fmap.rendering.VectorialLegend;
import com.iver.utiles.IPersistance;
import com.iver.utiles.XMLEntity;

public class FLayerGenericVectorial extends FLyrVect {
	private boolean loaded = false;

	private VectorialDriver vDriver = null;
	
	
	/* Esto deberia ir en el FLyrDefault */
	public void setProjectionByName(String projectionName) throws Exception{		
		IProjection proj = CRSFactory.getCRS(projectionName);
		if (proj == null) {
			throw new Exception("No se ha encontrado la proyeccion: "+ projectionName);
		}
		this.setProjection(proj);
		
	}

	
	public void setDriver(VectorialDriver vDriver) {
		this.vDriver = vDriver;
	}
	
	public VectorialDriver getDriver() {
		return this.vDriver ;
	}

	public void setDriverByName(String driverName) {
		this.setDriver(
		  (VectorialDriver)LayerFactory.getDM().getDriver(driverName) 
		);
	}

	
	/* Esto deberia ir en FLyrVect */
	private void initializeLegendDefault() throws FieldNotFoundException, DriverException {
		if (this.getLegend() == null) {
            if (this.getRecordset().getDriver() instanceof WithDefaultLegend) {
                WithDefaultLegend aux = (WithDefaultLegend) this.getRecordset().getDriver();
                this.setLegend((VectorialLegend) aux.getDefaultLegend());                                        
            } else {
                this.setLegend(LegendFactory.createSingleSymbolLegend(
                        this.getShapeType()));
            }
		}		
	}
	/* FIXME: esto tendria que tener declarado un throws de algo*/
	public void wakeUp() {
		if (!loaded) {
			try {
				this.load();
			} catch (DriverIOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
	}

	
	public void load() throws DriverIOException {
		if (this.getName() == null || this.getName().length() == 0) {
			this.setAvailable(false);
			//TODO: traducir???
			throw new DriverIOException("No se ha especificado nombre de capa");
		}
		if (this.vDriver == null) {
			this.setAvailable(false);
			//TODO: traducir???
			throw new DriverIOException("No se ha especificado driver");			
		}
		if (this.getProjection() == null) {
			this.setAvailable(false);
			//TODO: traducir???
			throw new DriverIOException("No se ha especificado proyeccion");			
		}
		
		VectorialAdapter adapter = null;
		try {
			adapter = new VectorialDefaultAdapter();
			adapter.setDriver((VectorialDriver) this.vDriver);
			this.setSource(adapter);
		} catch (Exception e) {
			this.setAvailable(false);
			throw new DriverIOException(e);
		}
		
		try {
			this.putLoadSelection();
			this.putLoadLegend();
			this.initializeLegendDefault();
			
		} catch (Exception e) {
			this.setAvailable(false);
			//TODO: traducir???
			throw new DriverIOException(e);
		}
		this.cleanLoadOptions();
	}

	public void setXMLEntity(XMLEntity xml) throws XMLException {
        IProjection proj = null;
        if (xml.contains("proj")) {
            proj = CRSFactory.getCRS(xml.getStringProperty("proj"));
        }
        else
        {
            proj = this.getMapContext().getViewPort().getProjection();
        }
		this.setName(xml.getName());
		this.setProjection(proj);
		
        String driverName = xml.getStringProperty("other");
        VectorialDriver driver = null;
        try {
            driver = (VectorialDriver) LayerFactory.getDM().getDriver(driverName);
        } catch (DriverLoadException e) {
            // Si no existe ese driver, no pasa nada.
            // Puede que el desarrollador no quiera que
            // aparezca en el cuadro de dilogo y ha metido
            // el jar con sus clases en nuestro directorio lib.
            // Intentamos cargar esa clase "a pelo".
            if (xml.getChild(2).contains("className"))
            {
                String className2 = xml.getChild(2).getStringProperty("className");
                try {
                    driver = (VectorialDriver) Class.forName(className2).newInstance();
                } catch (Exception e1) {
                    throw new XMLException(e1);
                }
            }
        } catch (NullPointerException npe) {
            // Si no existe ese driver, no pasa nada.
            // Puede que el desarrollador no quiera que
            // aparezca en el cuadro de dilogo y ha metido
            // el jar con sus clases en nuestro directorio lib.
            // Intentamos cargar esa clase "a pelo".
            if (xml.getChild(2).contains("className"))
            {
                String className2 = xml.getChild(2).getStringProperty("className");
                try {
                    driver = (VectorialDriver) Class.forName(className2).newInstance();
                } catch (Exception e1) {
                    throw new XMLException(e1);
                }
            }
        }
        if (driver == null) {
        	throw new XMLException(new Exception("Error al cargar el driver"));
        }
        if (driver instanceof IPersistance)
        {
            IPersistance persist = (IPersistance) driver;
            persist.setXMLEntity(xml.getChild(2));
        }
        this.setDriver(driver);
        super.setXMLEntityNew(xml);
	}
}
