/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package com.iver.cit.gvsig.fmap.layers;

import java.util.ArrayList;

import org.cresques.filter.RasterFilterStackManager;
import org.cresques.io.GeoRasterFile;

import com.iver.utiles.XMLEntity;

/**
 * Esta clase almacena el estado de un raster en cuanto a las caracteristicas
 * de opacidad, bandas y filtros. Estas caractersticas pueden ser salvadas a 
 * un xml y recuperadas por la capa a travs de las funciones setXMLEntity y 
 * getXMLEntity
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class StatusLayerRaster implements StatusRasterInterface{

	public static String 			defaultClass = "com.iver.cit.gvsig.fmap.layers.StatusLayerRaster";
	
	//Valor de opacidad global de la imagen
	public int						transparency = 0;
		
	//(Seleccin de bandas)Nmero de banda  asignado al Rojo, verde y azul 	
	public int 						bandR = 0;
	public int 						bandG = 1;
	public int 						bandB = 2;
	
	//Ficheros cargados en un PxRaster 
	public ArrayList				files = new ArrayList();
	
	//Filtros para poder montar una nueva pila
	public ArrayList				filters = new ArrayList();
		
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.StatusRasterInterface#setXMLEntity(com.iver.utiles.XMLEntity)
	 */
	public void setXMLEntity(XMLEntity xml, RasterOperations layer)throws XMLException {
	
		//Recuperamos las propiedades de los filtros
		for(int i=0;i<xml.getPropertyCount();i++){
			if(xml.getPropertyName(i).startsWith("filter."))	
				filters.add(xml.getPropertyName(i)+"="+xml.getPropertyValue(i));
		}
		
		if (xml.contains("raster.opacityLevel")) {
			transparency = xml.getIntProperty("raster.opacityLevel");
		}
		if (xml.contains("raster.bandR")) {
			bandR = xml.getIntProperty("raster.bandR");
		} 
		if (xml.contains("raster.bandG")) {
			bandG = xml.getIntProperty("raster.bandG");
		}
		if (xml.contains("raster.bandB")) {
			bandB = xml.getIntProperty("raster.bandB");
		}
	
		int cont = 0;
		while(true && cont<50){
			if (xml.contains("raster.file"+cont)) {
				files.add(xml.getStringProperty("raster.file"+cont));
				cont++;
			}else 
				break;
		}
	}
	

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.StatusRasterInterface#getXMLEntity(com.iver.utiles.XMLEntity)
	 */
	public void getXMLEntity(XMLEntity xml, boolean loadClass, RasterOperations layer) throws XMLException {
		
		if(loadClass)
			xml.putProperty("raster.class", StatusLayerRaster.defaultClass);
		
		if(transparency!=255)
			xml.putProperty("raster.opacityLevel", ""+transparency);
		
		xml.putProperty("raster.bandR", ""+bandR);
		xml.putProperty("raster.bandG", ""+bandG);
		xml.putProperty("raster.bandB", ""+bandB);
		
		
		if(files!=null && files.size()!=0){
			for(int i=0;i<files.size();i++)
				xml.putProperty("raster.file"+i, ""+((String)files.get(i)));
		}
		
		//Salvamos la pila de filtros.
		//Si la pila es null (esto puede ocurrir cuando se abre un 
		//proyecto que tiene WCS y no se abre la vista de este) entonces hay que leer los filtros
		//que van a salvarse a disco directamente de la variable filters que es la que se ha cargado
		//al hacer el setXMLEntity.
		
		RasterFilterStackManager stackManager = null;
		ArrayList l = null;
		if(layer.getFilterStack()!=null){
			stackManager = new RasterFilterStackManager(layer.getFilterStack());
			l = stackManager.getStringsFromStack();
			if(l == null || l.size() == 0)
				l = filters;
		}else
			l = filters;
					 
		for(int i=0;i<l.size();i++)
			xml.putProperty(getElem((String)l.get(i)), getValue((String)l.get(i)));
	}
	
	/**
	 * Obtiene el valor de una cadena de la forma elemento=valor
	 * @param cadena 
	 * @return
	 */
	private String getValue(String cadena){
		if(cadena!=null)
			return cadena.substring(cadena.indexOf("=")+1, cadena.length());
		else 
			return null;
	}
	
	/**
	 * Obtiene el elemento de una cadena de la forma elemento=valor
	 * @param cadena 
	 * @return
	 */
	private String getElem(String cadena){
		if(cadena!=null)
			return cadena.substring(0, cadena.indexOf("="));
		else 
			return null;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.StatusRasterInterface#getFilters()
	 */
	public ArrayList getFilters(){
		return this.filters;
	}
	
	
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.StatusRasterInterface#applyStatus(com.iver.cit.gvsig.fmap.layers.RasterFileAdapter)
	 */
	public void applyStatus(RasterOperations layer){
										
		//Eliminamos el fichero inicial y cargamos las bandas si hay para que se carguen 
		//en el orden correcto
		if(layer instanceof FLyrRaster){
			if(files.size()!=0){
				((FLyrRaster)layer).delFile((String)files.get(0));
				for(int i=0;i<files.size();i++)
					((FLyrRaster)layer).addFiles((String)files.get(i));
			}
		}
		
		//Asigna las bandas
		if(bandR==-1)bandR=0;
		if(bandG==-1)bandG=0;
		if(bandB==-1)bandB=0;
		layer.setBand(GeoRasterFile.RED_BAND, bandR);
		layer.setBand(GeoRasterFile.GREEN_BAND, bandG);
		layer.setBand(GeoRasterFile.BLUE_BAND, bandB);
		
		//Asigna la transparencia
		if(transparency != 255)
			layer.setTransparency(transparency);
		
		//Crea la pila de filtros a partir de las propiedades recuperadas desde el XML
		if(layer.getFilterStack() != null){
			RasterFilterStackManager stackManager = new RasterFilterStackManager(layer.getFilterStack());
			if(layer instanceof FLyrRaster)
				stackManager.createStackFromStrings(this.filters, ((FLyrRaster)layer).getSource().getFiles());
			else
				stackManager.createStackFromStrings(this.filters, new Integer(0));
		}
			
	}
}