/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.rendering;

import com.iver.cit.gvsig.fmap.layers.LegendEvent;


/**
 * Evento de cambio de leyenda.
 *
 * @author Vicente Caballero Navarro
 */
public class LegendChangedEvent extends LegendEvent {
	private static final int LEGEND_CHANGED = 0;
	
	private VectorialLegend oldLegend;
	private VectorialLegend newLegend;
	private LegendChangedEvent[] events;

	public static LegendChangedEvent createLegendChangedEvent(VectorialLegend oldLegend,
			VectorialLegend newLegend){
		return new LegendChangedEvent(oldLegend, newLegend, LEGEND_CHANGED);
	}
	
	/**
	 * Crea un nuevo LegendChangedEvent.
	 *
	 * @param oldLegend Antigua leyenda.
	 * @param newLegend Nueva leyenda.
	 */
	private LegendChangedEvent(VectorialLegend oldLegend,
		VectorialLegend newLegend, int eventType) {
		this.oldLegend = oldLegend;
		this.newLegend = newLegend;
		setEventType(eventType);
	}

	/**
	 * Devuelve la antigua leyenda.
	 *
	 * @return Antigua leyenda.
	 */
	public VectorialLegend getOldLegend() {
		return oldLegend;
	}

	/**
	 * Devuelve la nueva leyenda.
	 *
	 * @return Nueva leyenda.
	 */
	public VectorialLegend getNewLegend() {
		return newLegend;
	}

	/**
	 * Vector de LegendChangedEvent con los eventos.
	 *
	 * @return Vector de LegendChangedEvent.
	 */
	public LegendChangedEvent[] getEvents() {
		return events;
	}

	/**
	 * Inserta un vector con LegendChangedEvent.
	 *
	 * @param events Vector de LegendChangedEvent.
	 */
	public void setEvents(LegendChangedEvent[] events) {
		this.events = events;
	}
}
