/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.rendering.styling;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.Point2D;

import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.GeneralPathX;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.ShapeFactory;


/**
 * DOCUMENT ME!
 *
 * @author Vicente Caballero Navarro
 */
public class SymbolPreviewDrawer {
    private static IGeometry gPoint;
    private static IGeometry gLine;
    private static IGeometry gPolygon;
    private static IGeometry gText;
    FStyle2D fs2d;

    /**
     * Crea un nuevo SymbolPreviewDrawer.
     */
    public SymbolPreviewDrawer() {
    }

    /**
     * Crea un nuevo PreviousSymbolDrawer.
     *
     * @param f DOCUMENT ME!
     */
    public SymbolPreviewDrawer(FStyle2D f) {
        fs2d = f;
    }

    /**
     * DOCUMENT ME!
     *
     * @param g DOCUMENT ME!
     * @param rec DOCUMENT ME!
     * @param type DOCUMENT ME!
     */
    public void draw(Graphics2D g, Rectangle rec, int type) {
        boolean point = false;
        boolean line = false;
        boolean polygon = false;
        boolean text = false;
        int all = 0;

        System.out.println("type = " + type);

        if ((type % 2) > 0) { //POINT
            point = true;
            all++;
        }

        type = type / 2;

        if ((type % 2) > 0) { //LINE
            line = true;
            all++;
        }

        type = type / 2;

        if ((type % 2) > 0) { //POLYGON
            polygon = true;
            all++;
        }

        type = type / 2;

        if ((type % 2) > 0) { //TEXT
            text = true;
            all++;
        }

        double x = rec.getX();
        double y = rec.getY();
        double w = rec.getWidth();
        double h = rec.getHeight();
        x = x + (w / 1.3);
        y = y + (h / 4);
        w = w / 5;
        h = h / 2;

        double part = w / all;

        Rectangle[] symbolRect = new Rectangle[all];

        for (int i = 0; i < all; i++) {
            symbolRect[i] = new Rectangle((int) (x + (part * i)), (int) y,
                    (int) (part), (int) h);
        }

        int num = 0;

        if (point) {
            drawPoint(g, symbolRect[num++]);
        }

        if (line) {
            drawLine(g, symbolRect[num++]);
        }

        if (polygon) {
            drawPolygon(g, symbolRect[num++]);
        }

        if (text) {
            drawText(g, symbolRect[num++]);
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param rec DOCUMENT ME!
     */
    private void initPoint(Rectangle rec) {
        gPoint = ShapeFactory.createPoint2D(rec.getCenterX(), rec.getCenterY());
    }

    /**
     * DOCUMENT ME!
     *
     * @param rec DOCUMENT ME!
     */
    private void initLine(Rectangle rec) {
        Point2D[] points = new Point2D.Double[4];
        int x = (int) rec.getX();
        int y = (int) rec.getY();
        int height = (int) rec.getHeight();
        int width = (int) rec.getWidth();
        points[0] = new Point2D.Double(x, y + (height / 2));
        points[1] = new Point2D.Double(x + (width / 3), y + height);
        points[2] = new Point2D.Double(x + ((2 * width) / 3), y);
        points[3] = new Point2D.Double(x + width, y + (height / 2));

        int[] parts = new int[1];
        parts[0] = 0;

        GeneralPathX gPX = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD,
                points.length);
        initGeneralPathX(gPX, points, parts);
        gLine = ShapeFactory.createPolyline2D(gPX);
    }

    /**
     * DOCUMENT ME!
     *
     * @param rec DOCUMENT ME!
     */
    private void initPolygon(Rectangle rec) {
        Point2D[] points = new Point2D.Double[5];
        int x = (int) rec.getX();
        int y = (int) rec.getY();
        int height = (int) rec.getHeight();
        int width = (int) rec.getWidth();

        points[0] = new Point2D.Double(x, y);
        points[1] = new Point2D.Double(rec.getMaxX(), y);
        points[2] = new Point2D.Double(rec.getMaxX(), rec.getMaxY());
        points[3] = new Point2D.Double(x, rec.getMaxY());
        points[4] = new Point2D.Double(x, y);

        int[] parts = new int[1];
        parts[0] = 0;

        GeneralPathX gPX = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD,
                points.length);
        initGeneralPathX(gPX, points, parts);
        gPolygon = ShapeFactory.createPolygon2D(gPX);
    }

    /**
     * DOCUMENT ME!
     *
     * @param rec DOCUMENT ME!
     */
    private void initText(Rectangle rec) {
    	// TODO:

    }

    /**
     * DOCUMENT ME!
     *
     * @param gPX DOCUMENT ME!
     * @param po DOCUMENT ME!
     * @param pa DOCUMENT ME!
     */
    private void initGeneralPathX(GeneralPathX gPX, Point2D[] po, int[] pa) {
        int j = 0;

        for (int i = 0; i < po.length; i++) {
            if (i == pa[j]) {
                gPX.moveTo(po[i].getX(), po[i].getY());

                if (j < (pa.length - 1)) {
                    j++;
                }
            } else {
                gPX.lineTo(po[i].getX(), po[i].getY());
            }
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param g DOCUMENT ME!
     * @param rect DOCUMENT ME!
     */
    public void drawPoint(Graphics2D g, Rectangle rect) {
        initPoint(rect);

        ViewPort vp = new ViewPort(null);
        vp.setOffset(new Point2D.Double(rect.getX(), rect.getY()));
        vp.setImageSize(new Dimension((int) rect.getWidth(),
                (int) rect.getHeight()));
        vp.setExtent(rect);
        gPoint.draw(g, vp, fs2d);
    }

    /**
     * DOCUMENT ME!
     *
     * @param g DOCUMENT ME!
     * @param rect DOCUMENT ME!
     */
    public void drawLine(Graphics2D g, Rectangle rect) {
        initLine(rect);

        ViewPort vp = new ViewPort(null);
        vp.setOffset(new Point2D.Double(rect.getX(), rect.getY()));
        vp.setImageSize(new Dimension((int) rect.getWidth(),
                (int) rect.getHeight()));
        vp.setExtent(rect);
        gLine.draw(g, vp, fs2d);
    }

    /**
     * DOCUMENT ME!
     *
     * @param g DOCUMENT ME!
     * @param rect DOCUMENT ME!
     */
    public void drawPolygon(Graphics2D g, Rectangle rect) {
        initPolygon(rect);

        ViewPort vp = new ViewPort(null);
        vp.setOffset(new Point2D.Double(rect.getX(), rect.getY()));
        vp.setImageSize(new Dimension((int) rect.getWidth(),
                (int) rect.getHeight()));
        vp.setExtent(rect);
        gPolygon.draw(g, vp, fs2d);
    }

    /**
     * DOCUMENT ME!
     *
     * @param g DOCUMENT ME!
     * @param rect DOCUMENT ME!
     */
    public void drawText(Graphics2D g, Rectangle rect) {
        initText(rect);

        ViewPort vp = new ViewPort(null);
        vp.setOffset(new Point2D.Double(rect.getX(), rect.getY()));
        vp.setImageSize(new Dimension((int) rect.getWidth(),
                (int) rect.getHeight()));
        vp.setExtent(rect);
        gText.draw(g, vp, fs2d);
    }

    /**
     * DOCUMENT ME!
     *
     * @param fs2d DOCUMENT ME!
     */
    public void setSymbol(FStyle2D fs2d) {
        this.fs2d = fs2d;
    }
}
