package com.hardcode.gdbms.engine.data.indexes;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;


/**
 * Implementacin de FixedIndexSet que escribe los ndices en un fichero
 *
 * @author Fernando Gonzlez Corts
 */
public class FixedDiskIndexSet extends DiskIndexSet implements FixedIndexSet {
	private long size = 0;

	/**
	 * Crea un nuevo FixedDiskIndexSet.
	 *
	 * @param size DOCUMENT ME!
	 */
	public FixedDiskIndexSet(long size) {
		this.size = size;
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.indexes.VariableIndexSet#getIndex(long)
	 */
	public long getIndex(long nth) throws IOException {
		buffer.clear();
		inputChannel.read(buffer, nth * 8);
		buffer.flip();

		return buffer.getLong();
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.indexes.VariableIndexSet#getIndexCount()
	 */
	public long getIndexCount() {
		return size;
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.indexes.VariableIndexSet#open(java.io.File)
	 */
	public void open(File f) throws IOException {
		file = f;
		fos = new FileOutputStream(f);
		outputChannel = fos.getChannel();
		fis = new FileInputStream(file);
		inputChannel = fis.getChannel();
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.indexes.VariableIndexSet#close()
	 */
	public void close() throws IOException {
		inputChannel.close();
		fis.close();
		outputChannel.close();
		fos.close();
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.indexes.FixedIndexSet#setIndex(long,
	 * 		long)
	 */
	public void setIndex(long index, long value) throws IOException {
		buffer.put((byte) (index >>> 56));
		buffer.put((byte) (index >>> 48));
		buffer.put((byte) (index >>> 40));
		buffer.put((byte) (index >>> 32));
		buffer.put((byte) (index >>> 24));
		buffer.put((byte) (index >>> 16));
		buffer.put((byte) (index >>> 8));
		buffer.put((byte) (index >>> 0));

		buffer.flip();

		outputChannel.write(buffer, index * 8);

		buffer.clear();

		size++;
	}
}
