package org.gvsig.gui.beans.comboBoxItemsSeeker;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.TreeSet;
import java.util.Vector;


/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */


/**
 * This class inherits of 'AbstractDefaultComboBoxItemsSeekerConfigurableModel' and it's optimized for not repetead items: and if there are repeated
 *    items this would it will work badly; if there are repeated items use 'ComboBoxItemsSeekerConfigurableModel'.  
 *    
 * Limitations: This model works badly if has repeated items.
 * 
 *  @author Pablo Piqueras Bartolom (p_queras@hotmail.com)
 */
public class ComboBoxSingularItemsSeekerConfigurableModel extends AbstractDefaultComboBoxItemsSeekerConfigurableModel implements java.io.Serializable {
	private static final long serialVersionUID = -2442496446788716549L;
	
	// INNER MODEL STRUCTURES
	private TreeSet itemsShowed;
	// END INNER MODEL STRUCTURES
	
	// OTHER FLAGS
	protected boolean thisIsAComboBoxSingularItemsSeekerConfigurableModelClassInialized = false; // To distinguish the class which invokes some methods of this class
	// END OTHER FLAGS
	
	/**
	 * Default constructor without parameters
	 * 
	 * @see AbstractDefaultComboBoxItemsSeekerConfigurableModel#AbstractDefaultComboBoxItemsSeekerConfigurableModel() 
	 */
	public ComboBoxSingularItemsSeekerConfigurableModel() {
		super();
		this.initialize();
	}

	/**
	 * Default constructor with an array as a parameter
	 * 
	 * @see AbstractDefaultComboBoxItemsSeekerConfigurableModel#AbstractDefaultComboBoxItemsSeekerConfigurableModel(Object[]) 
	 */
	public ComboBoxSingularItemsSeekerConfigurableModel(Object[] items) {
		super(items);
		this.initialize();
		this.itemsShowed.addAll(Arrays.asList(items));
	}

	/**
	 * Default constructor with a Vector as a parameter
	 * 
	 * @see AbstractDefaultComboBoxItemsSeekerConfigurableModel#AbstractDefaultComboBoxItemsSeekerConfigurableModel(Vector) 
	 */
	public ComboBoxSingularItemsSeekerConfigurableModel(Vector objects) {
		super(objects);
		this.initialize();
		this.itemsShowed.addAll(objects);
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gui.beans.comboBoxItemsSeeker.AbstractDefaultComboBoxItemsSeekerConfigurableModel#initialize()
	 */
	protected void initialize() {
		super.initialize();
		logger.debug("Se ejecuta el initialize del hijo");
		
		thisIsAComboBoxSingularItemsSeekerConfigurableModelClassInialized = true;
		this.itemsShowed = new TreeSet();
	}
	
	
	
	////// REIMPLEMENTATION OF SOME METHODS OF "DefaultComboBoxModel" TO ADAPT THE BEHAVIOR OF THIS MODEL //////
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.comboBoxItemsSeeker.AbstractDefaultComboBoxItemsSeekerConfigurableModel#addElement(java.lang.Object)
	 */
	public void addElement(Object anObject) {
		super.addElementToParent(anObject); // this adds the element and selects by default the first item added
		
		// Add the item also in the inner attribute
		this.itemsShowed.add(anObject);

		this.reset();
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.comboBoxItemsSeeker.AbstractDefaultComboBoxItemsSeekerConfigurableModel#getElementAt(int)
	 */
	public Object getElementAt(int index) {
		if (!this.thisIsAComboBoxSingularItemsSeekerConfigurableModelClassInialized)
			return super.getParentElementAt(index);
		
		if (this.startState) // At the beginning shows all items
		{			
			switch(this.getStartBehavior())
			{
				case MAINTAIN_ORIGINAL_POSITION_START :				
					return super.getParentElementAt(index);
				case ORDERED_START :
					// If the items aren't sorted -> sort them!
					if ((!this.itemsInTheArray) || (this.stateAllItems != STATE_ITEMS_ARE_ORDERED))
						this.setItemsCopyToArray();
					
					// Return the item at that position
					return this.itemsSearchesShow[index];
				case DISORDERED_START :
					
					if (!this.itemsInTheArray)
						this.setItemsCopyToArray();
					
					// If the items aren't disordered -> sort them!
					if (this.stateAllItems != STATE_ITEMS_ARE_DISORDERED)
						this.disorderItemsCopy();

					// Return the item at that position
					return this.itemsSearchesShow[index];
				default:
					// If error configuration -> it gives back an empty object for not to produce more errors 
					return new Object();
			}
		}
		else
		{			
			switch(this.getSearchBehavior())
			{
				case MAINTAIN_ORIGINAL_POSITION_ALL_ITEMS_SEARCH :
					return super.getParentElementAt(index);
				case ORDERED_ALL_ITEMS_SEARCH :
					if ((!this.itemsInTheArray) || (this.stateAllItems != STATE_ITEMS_ARE_ORDERED))
						this.setItemsCopyToArray();
					return this.itemsSearchesShow[index];
				case DISORDERED_ALL_ITEMS_SEARCH :
					if (!this.itemsInTheArray)
						this.setItemsCopyToArray();

					// If the items aren't sorted -> sort them!
					if (this.stateAllItems != STATE_ITEMS_ARE_DISORDERED)
						this.disorderItemsCopy();
					return this.itemsSearchesShow[index];
				case MAINTAIN_ORIGINAL_POSITION_DYNAMIC_SEARCH : case ORDERED_DYNAMIC_SEARCH : case DISORDERED_DYNAMIC_SEARCH :
					if (!this.itemsInTheArray)
						this.setItemsCopyToArray();

					if (!this.dynamicSearchDone)
						seekDynamicItems();
					
					return this.itemsSearchesShow[index];
				default:
					// If error configuration -> it gives back an empty object for not to produce more errors 
					return new Object();
			}
		}		
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.comboBoxItemsSeeker.AbstractDefaultComboBoxItemsSeekerConfigurableModel#getIndexOf(java.lang.Object)
	 */
	public int getIndexOf(Object anObject) {
		if (!this.thisIsAComboBoxSingularItemsSeekerConfigurableModelClassInialized)
			return super.getParentIndexOf(anObject);
		
		if (!this.itemsInTheArray)
		{
			if (this.startState) // At the beginning shows all items
			{			
				switch(this.getStartBehavior())
				{
					case MAINTAIN_ORIGINAL_POSITION_START : // Do nothing	
						break;
					case ORDERED_START :
						this.setItemsCopyToArray();
						break;
					case DISORDERED_START :
						if (!this.itemsInTheArray)
							this.setItemsCopyToArray();
						
						// If the items aren't disordered -> sort them!
						if (this.stateAllItems != STATE_ITEMS_ARE_DISORDERED)
							this.disorderItemsCopy();

						break;
					default:
						// If error configuration -> it gives back -1 for not to produce more errors 
						return -1;
				}
			}
			else
			{			
				switch(this.getSearchBehavior())
				{
					case MAINTAIN_ORIGINAL_POSITION_ALL_ITEMS_SEARCH : // Do nothing	
						break;
					case ORDERED_ALL_ITEMS_SEARCH :
						if ((!this.itemsInTheArray) || (this.stateAllItems != STATE_ITEMS_ARE_ORDERED))
							this.setItemsCopyToArray();
						break;
					case DISORDERED_ALL_ITEMS_SEARCH :
						if (!this.itemsInTheArray)
							this.setItemsCopyToArray();

						// If the items aren't disordered -> sort them!
						if (this.stateAllItems != STATE_ITEMS_ARE_DISORDERED)
							this.disorderItemsCopy();

						break;
					case MAINTAIN_ORIGINAL_POSITION_DYNAMIC_SEARCH : case ORDERED_DYNAMIC_SEARCH : case DISORDERED_DYNAMIC_SEARCH :
						if (!this.itemsInTheArray)
							this.setItemsCopyToArray();

						if (!this.dynamicSearchDone)
							seekDynamicItems();
						
						break;
					default:
						// If error configuration -> it gives back -1 for not to produce more errors 
						return -1;
				}
			}		
			
		}

		if (((this.startState) && (this.getStartBehavior() == MAINTAIN_ORIGINAL_POSITION_START)) || ((!this.startState) && (this.getSearchBehavior() == MAINTAIN_ORIGINAL_POSITION_ALL_ITEMS_SEARCH)))
			return super.getParentIndexOf(anObject); //this.itemsShowed.indexOf(anObject);
		else
			return (Arrays.asList(this.itemsSearchesShow)).indexOf(anObject);

	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.comboBoxItemsSeeker.AbstractDefaultComboBoxItemsSeekerConfigurableModel#getSize()
	 */
	public int getSize() {
		if (!this.thisIsAComboBoxSingularItemsSeekerConfigurableModelClassInialized)
			return super.getParentSize();
		
		if (this.startState)
			return this.itemsShowed.size();			
		
		switch(this.getSearchBehavior())
		{
			case MAINTAIN_ORIGINAL_POSITION_ALL_ITEMS_SEARCH: case ORDERED_ALL_ITEMS_SEARCH: case DISORDERED_ALL_ITEMS_SEARCH:
				return this.itemsShowed.size();
			case MAINTAIN_ORIGINAL_POSITION_DYNAMIC_SEARCH: case ORDERED_DYNAMIC_SEARCH: case DISORDERED_DYNAMIC_SEARCH:
				if (!this.itemsInTheArray)
					this.setItemsCopyToArray();

				if (!this.dynamicSearchDone)
					seekDynamicItems();

				return this.itemsSearchesShow.length;
//			case ORDERED_DYNAMIC_SEARCH:
//				if (!this.dynamicSearchDone)
//					seekDynamicItems();
//
//				return this.itemsSearchesShow.length;
//			case DISORDERED_DYNAMIC_SEARCH:
//				if (!this.dynamicSearchDone)
//					seekDynamicItems();
//
//				return this.itemsSearchesShow.length;
			default:
				// If error configuration -> it gives back 0 for not to produce more errors 
				return 0;
		}
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.comboBoxItemsSeeker.AbstractDefaultComboBoxItemsSeekerConfigurableModel#insertElementAt(java.lang.Object, int)
	 */
	public void insertElementAt(Object anObject, int index) {
		super.insertToParentElementAt(anObject, index);
		
		// Add the item also in the inner attribute
		this.itemsShowed.add(anObject);

		this.reset();		
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.comboBoxItemsSeeker.AbstractDefaultComboBoxItemsSeekerConfigurableModel#removeAllElements()
	 */
	public void removeAllElements() {
		super.removeAllElementsOfParent();
		
		// Also empty the inner attribute
		itemsShowed.clear();		
		
		this.reset();	
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.comboBoxItemsSeeker.AbstractDefaultComboBoxItemsSeekerConfigurableModel#removeElement(java.lang.Object)
	 */
	public void removeElement(Object anObject) {
		super.removeParentElement(anObject);
		
		// Remove the object also in the inner attribute
		itemsShowed.remove(anObject);
		
		this.reset();		
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.comboBoxItemsSeeker.AbstractDefaultComboBoxItemsSeekerConfigurableModel#removeElementAt(int)
	 */
	public void removeElementAt(int index) {
		Object obj = new Object();
		
		obj = super.getParentElementAt(index);
		super.removeParentElementAt(index);
		
		// Remove the object also in the inner attribute
		itemsShowed.remove(obj);
		
		this.reset();
	}

	////// END REIMPLEMENTATION OF SOME METHODS OF "DefaultComboBoxModel" TO ADAPT THE BEHAVIOR OF THIS MODEL //////	

	
	
	////// OTHER METHODS //////
	
	/*
	 *  (non-Javadoc)
	 * @see AbstractDefaultComboBoxItemsSeekerConfigurableModel#getParentAllData()
	 */
	public Vector getParentAllData()
	{
		Vector data = new Vector();
		int size;
		
		logger.debug("Parent size: " + super.getParentSize());
		logger.debug("ItemsShowed size: " + this.itemsShowed.size());
		
		// Caso de model que no permite items repetidos con items repetidos
		if (this.itemsShowed.size() < super.getParentSize())
			size = super.getParentSize();
		else
			size = this.itemsShowed.size();
		
		for (int i = 0; i < size; i++)
			data.add(super.getParentElementAt(i));
		
		return data;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.comboBoxItemsSeeker.AbstractDefaultComboBoxItemsSeekerConfigurableModel#disorderItemsCopy()
	 */
	protected void disorderItemsCopy() {
		if (this.getSearchBehavior() == DISORDERED_DYNAMIC_SEARCH)
			Collections.shuffle(Arrays.asList(this.itemsSearchesShow)); // Disorder the items to show
		else
		{
			this.itemsSearchesShow = this.itemsShowed.toArray();
			Collections.shuffle(Arrays.asList(this.itemsSearchesShow)); // Disorder the items
		}
		
		this.stateAllItems = STATE_ITEMS_ARE_DISORDERED;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.comboBoxItemsSeeker.AbstractDefaultComboBoxItemsSeekerConfigurableModel#getDynamicItemsSearch(org.gvsig.gui.beans.comboBoxItemsSeeker.AbstractDefaultComboBoxItemsSeekerConfigurableModel.BinaryRankOfItemsSearch, int, int)
	 */
	protected void getDynamicItemsSearch(BinaryRankOfItemsSeek search, int lowIndex, int hightIndex) {
		int lowIndexSubSet2;
		int hightIndexSubSet2;
		List aux;

		if (isCaseSensitive_Flag())
		{
			if (lowIndex <= hightIndex)
			{
				if (hightIndex == (itemsSearchesShow.length - 1))
				{
					aux = new Vector(hightIndex - lowIndex + 1 , 1); // Parameters: Initial Capacity and 1 -> capacity increment
					aux.addAll(Arrays.asList((this.itemsShowed.subSet(this.itemsSearchesShow[lowIndex], this.itemsSearchesShow[hightIndex])).toArray()));
					aux.add(this.itemsSearchesShow[hightIndex]);
					this.itemsSearchesShow = aux.toArray();
				}
				else
					this.itemsSearchesShow = (this.itemsShowed.subSet(this.itemsSearchesShow[lowIndex], this.itemsSearchesShow[hightIndex+1])).toArray();
			}
			else // Any items match their start with the written text
				this.itemsSearchesShow = new Object[0];
		}
		else
		{
			lowIndexSubSet2 = search.getLowIndexSecondSubSet();
			hightIndexSubSet2 = search.getHightIndexSecondSubSet();
			
			// If any element has been found
			if ((lowIndex > hightIndex) && (lowIndexSubSet2 > hightIndexSubSet2))
			{
				this.itemsSearchesShow = new Object[0];
				return;
			}

			aux = new Vector();
			
			// Second Subset (Uppercases are first)
			if (lowIndexSubSet2 <= hightIndexSubSet2)
			{
				if ((lowIndex != lowIndexSubSet2) && (hightIndex != hightIndexSubSet2))
				{
					aux.addAll(Arrays.asList((this.itemsShowed.subSet(this.itemsSearchesShow[lowIndexSubSet2], this.itemsSearchesShow[hightIndexSubSet2])).toArray()));
					aux.add(this.itemsSearchesShow[hightIndexSubSet2]);
				}
			}
	
			// First Subset (Uppercases are first)
			if (lowIndex <= hightIndex)
			{
				aux.addAll(Arrays.asList((this.itemsShowed.subSet(this.itemsSearchesShow[lowIndex], this.itemsSearchesShow[hightIndex])).toArray()));
				aux.add(this.itemsSearchesShow[hightIndex]);
			}
			
			this.itemsSearchesShow = aux.toArray();
		}		
	}


	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.comboBoxItemsSeeker.AbstractDefaultComboBoxItemsSeekerConfigurableModel#seekDynamicItems()
	 */
	protected void seekDynamicItems() {
		int lowIndex;
		int hightIndex;
		
		// If doesn't have to seek items -> finish
		if (this.writtenText.compareTo("") == 0)
			return;
		
		if (isDynamicSearch()) // Puede que sobre este if
		{			
			// Get a sorted copy of all items
			this.itemsSearchesShow = this.itemsShowed.toArray();
			
			// Do the search of the rank
			BinaryRankOfItemsSeek search = new BinaryRankOfItemsSeek();
			search.binaryRankSeek();
						
			// To obtain the range of items to showing
			lowIndex = search.getLowIndex();
			hightIndex = search.getHightIndex(); 			
			
			this.dynamicSearchDone = true;
			
			// Get the items
			switch(this.getSearchBehavior())
			{
				case MAINTAIN_ORIGINAL_POSITION_DYNAMIC_SEARCH: // If maintain original position // REVISAR y HACER					
					int numberOfItems = hightIndex - lowIndex;
					
					this.getDynamicItemsSearch(search, lowIndex, hightIndex);
					
					if (numberOfItems > 0)
					{						
						int positions[] = new int[numberOfItems];
						
						int positionArrayIndex = 0;
						
						// Gets the items ordered
						this.getDynamicItemsSearch(search, lowIndex, hightIndex);
						
						// Gets the position of the items in the original position
						for (int i = lowIndex; i < hightIndex; i++)
						{
							positions[positionArrayIndex] = super.getParentIndexOf(this.itemsSearchesShow[i]);
							positionArrayIndex ++;
						}
						Arrays.sort(positions);
						
						this.itemsSearchesShow = new Object[numberOfItems]; // Remove the array
		
						// Get the items at the original position
						for (int i = 0; i < numberOfItems; i++)
							this.itemsSearchesShow[i] = super.getParentElementAt(positions[i]);	
					}
					break;
				case ORDERED_DYNAMIC_SEARCH: // If ordered			
					
					this.getDynamicItemsSearch(search, lowIndex, hightIndex);
					
					break;
				case DISORDERED_DYNAMIC_SEARCH: // If disordered //HACER				

					this.getDynamicItemsSearch(search, lowIndex, hightIndex);
					this.disorderItemsCopy();
			}			
		}
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.comboBoxItemsSeeker.AbstractDefaultComboBoxItemsSeekerConfigurableModel#setItemsCopyToArray()
	 */
	protected void setItemsCopyToArray() {
		this.itemsSearchesShow = null;
		this.itemsSearchesShow = this.itemsShowed.toArray();
		this.itemsInTheArray = true;
		this.stateAllItems = STATE_ITEMS_ARE_ORDERED;
	}

	////// END OTHER METHODS //////
}
